#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
from enum import StrEnum

# Do not edit this file. It is generated with tools/openepd/codegen/generate_geography_enum.py, or `make codegen`


class Geography(StrEnum):
    """
    Geography definitions.

    Special geography regions:

     * EU27: 27 members of the Euro bloc
     * NAFTA: North American Free Trade Agreement

    Country geography regions, see https://en.wikipedia.org/wiki/ISO_3166-1

     * AD: Andorra
     * AE: United Arab Emirates
     * AF: Afghanistan
     * AG: Antigua and Barbuda
     * AI: Anguilla
     * AL: Albania
     * AM: Armenia
     * AO: Angola
     * AQ: Antarctica
     * AR: Argentina
     * AS: American Samoa
     * AT: Austria
     * AU: Australia
     * AW: Aruba
     * AX: Åland Islands
     * AZ: Azerbaijan
     * BA: Bosnia and Herzegovina
     * BB: Barbados
     * BD: Bangladesh
     * BE: Belgium
     * BF: Burkina Faso
     * BG: Bulgaria
     * BH: Bahrain
     * BI: Burundi
     * BJ: Benin
     * BL: Saint Barthélemy
     * BM: Bermuda
     * BN: Brunei Darussalam
     * BO: Bolivia, Plurinational State of
     * BQ: Bonaire, Sint Eustatius and Saba
     * BR: Brazil
     * BS: Bahamas
     * BT: Bhutan
     * BV: Bouvet Island
     * BW: Botswana
     * BY: Belarus
     * BZ: Belize
     * CA: Canada
     * CC: Cocos (Keeling) Islands
     * CD: Congo, The Democratic Republic of the
     * CF: Central African Republic
     * CG: Congo
     * CH: Switzerland
     * CI: Côte d'Ivoire
     * CK: Cook Islands
     * CL: Chile
     * CM: Cameroon
     * CN: China
     * CO: Colombia
     * CR: Costa Rica
     * CU: Cuba
     * CV: Cabo Verde
     * CW: Curaçao
     * CX: Christmas Island
     * CY: Cyprus
     * CZ: Czechia
     * DE: Germany
     * DJ: Djibouti
     * DK: Denmark
     * DM: Dominica
     * DO: Dominican Republic
     * DZ: Algeria
     * EC: Ecuador
     * EE: Estonia
     * EG: Egypt
     * EH: Western Sahara
     * ER: Eritrea
     * ES: Spain
     * ET: Ethiopia
     * FI: Finland
     * FJ: Fiji
     * FK: Falkland Islands (Malvinas)
     * FM: Micronesia, Federated States of
     * FO: Faroe Islands
     * FR: France
     * GA: Gabon
     * GB: United Kingdom
     * GD: Grenada
     * GE: Georgia
     * GF: French Guiana
     * GG: Guernsey
     * GH: Ghana
     * GI: Gibraltar
     * GL: Greenland
     * GM: Gambia
     * GN: Guinea
     * GP: Guadeloupe
     * GQ: Equatorial Guinea
     * GR: Greece
     * GS: South Georgia and the South Sandwich Islands
     * GT: Guatemala
     * GU: Guam
     * GW: Guinea-Bissau
     * GY: Guyana
     * HK: Hong Kong
     * HM: Heard Island and McDonald Islands
     * HN: Honduras
     * HR: Croatia
     * HT: Haiti
     * HU: Hungary
     * ID: Indonesia
     * IE: Ireland
     * IL: Israel
     * IM: Isle of Man
     * IN: India
     * IO: British Indian Ocean Territory
     * IQ: Iraq
     * IR: Iran, Islamic Republic of
     * IS: Iceland
     * IT: Italy
     * JE: Jersey
     * JM: Jamaica
     * JO: Jordan
     * JP: Japan
     * KE: Kenya
     * KG: Kyrgyzstan
     * KH: Cambodia
     * KI: Kiribati
     * KM: Comoros
     * KN: Saint Kitts and Nevis
     * KP: Korea, Democratic People's Republic of
     * KR: Korea, Republic of
     * KW: Kuwait
     * KY: Cayman Islands
     * KZ: Kazakhstan
     * LA: Lao People's Democratic Republic
     * LB: Lebanon
     * LC: Saint Lucia
     * LI: Liechtenstein
     * LK: Sri Lanka
     * LR: Liberia
     * LS: Lesotho
     * LT: Lithuania
     * LU: Luxembourg
     * LV: Latvia
     * LY: Libya
     * MA: Morocco
     * MC: Monaco
     * MD: Moldova, Republic of
     * ME: Montenegro
     * MF: Saint Martin (French part)
     * MG: Madagascar
     * MH: Marshall Islands
     * MK: North Macedonia
     * ML: Mali
     * MM: Myanmar
     * MN: Mongolia
     * MO: Macao
     * MP: Northern Mariana Islands
     * MQ: Martinique
     * MR: Mauritania
     * MS: Montserrat
     * MT: Malta
     * MU: Mauritius
     * MV: Maldives
     * MW: Malawi
     * MX: Mexico
     * MY: Malaysia
     * MZ: Mozambique
     * NA: Namibia
     * NC: New Caledonia
     * NE: Niger
     * NF: Norfolk Island
     * NG: Nigeria
     * NI: Nicaragua
     * NL: Netherlands
     * NO: Norway
     * NP: Nepal
     * NR: Nauru
     * NU: Niue
     * NZ: New Zealand
     * OM: Oman
     * PA: Panama
     * PE: Peru
     * PF: French Polynesia
     * PG: Papua New Guinea
     * PH: Philippines
     * PK: Pakistan
     * PL: Poland
     * PM: Saint Pierre and Miquelon
     * PN: Pitcairn
     * PR: Puerto Rico
     * PS: Palestine, State of
     * PT: Portugal
     * PW: Palau
     * PY: Paraguay
     * QA: Qatar
     * RE: Réunion
     * RO: Romania
     * RS: Serbia
     * RU: Russian Federation
     * RW: Rwanda
     * SA: Saudi Arabia
     * SB: Solomon Islands
     * SC: Seychelles
     * SD: Sudan
     * SE: Sweden
     * SG: Singapore
     * SH: Saint Helena, Ascension and Tristan da Cunha
     * SI: Slovenia
     * SJ: Svalbard and Jan Mayen
     * SK: Slovakia
     * SL: Sierra Leone
     * SM: San Marino
     * SN: Senegal
     * SO: Somalia
     * SR: Suriname
     * SS: South Sudan
     * ST: Sao Tome and Principe
     * SV: El Salvador
     * SX: Sint Maarten (Dutch part)
     * SY: Syrian Arab Republic
     * SZ: Eswatini
     * TC: Turks and Caicos Islands
     * TD: Chad
     * TF: French Southern Territories
     * TG: Togo
     * TH: Thailand
     * TJ: Tajikistan
     * TK: Tokelau
     * TL: Timor-Leste
     * TM: Turkmenistan
     * TN: Tunisia
     * TO: Tonga
     * TR: Türkiye
     * TT: Trinidad and Tobago
     * TV: Tuvalu
     * TW: Taiwan, Province of China
     * TZ: Tanzania, United Republic of
     * UA: Ukraine
     * UG: Uganda
     * UM: United States Minor Outlying Islands
     * US: United States
     * UY: Uruguay
     * UZ: Uzbekistan
     * VA: Holy See (Vatican City State)
     * VC: Saint Vincent and the Grenadines
     * VE: Venezuela, Bolivarian Republic of
     * VG: Virgin Islands, British
     * VI: Virgin Islands, U.S.
     * VN: Viet Nam
     * VU: Vanuatu
     * WF: Wallis and Futuna
     * WS: Samoa
     * YE: Yemen
     * YT: Mayotte
     * ZA: South Africa
     * ZM: Zambia
     * ZW: Zimbabwe

    USA states and Canada provinces, see https://en.wikipedia.org/wiki/ISO_3166-1:

     * CA-AB: Alberta, Canada
     * CA-BC: British Columbia, Canada
     * CA-MB: Manitoba, Canada
     * CA-NB: New Brunswick, Canada
     * CA-NL: Newfoundland and Labrador, Canada
     * CA-NS: Nova Scotia, Canada
     * CA-NT: Northwest Territories, Canada
     * CA-NU: Nunavut, Canada
     * CA-ON: Ontario, Canada
     * CA-PE: Prince Edward Island, Canada
     * CA-QC: Quebec, Canada
     * CA-SK: Saskatchewan, Canada
     * CA-YT: Yukon, Canada
     * US-AK: Alaska, United States
     * US-AL: Alabama, United States
     * US-AR: Arkansas, United States
     * US-AS: American Samoa, United States
     * US-AZ: Arizona, United States
     * US-CA: California, United States
     * US-CO: Colorado, United States
     * US-CT: Connecticut, United States
     * US-DC: District of Columbia, United States
     * US-DE: Delaware, United States
     * US-FL: Florida, United States
     * US-GA: Georgia, United States
     * US-GU: Guam, United States
     * US-HI: Hawaii, United States
     * US-IA: Iowa, United States
     * US-ID: Idaho, United States
     * US-IL: Illinois, United States
     * US-IN: Indiana, United States
     * US-KS: Kansas, United States
     * US-KY: Kentucky, United States
     * US-LA: Louisiana, United States
     * US-MA: Massachusetts, United States
     * US-MD: Maryland, United States
     * US-ME: Maine, United States
     * US-MI: Michigan, United States
     * US-MN: Minnesota, United States
     * US-MO: Missouri, United States
     * US-MP: Northern Mariana Islands, United States
     * US-MS: Mississippi, United States
     * US-MT: Montana, United States
     * US-NC: North Carolina, United States
     * US-ND: North Dakota, United States
     * US-NE: Nebraska, United States
     * US-NH: New Hampshire, United States
     * US-NJ: New Jersey, United States
     * US-NM: New Mexico, United States
     * US-NV: Nevada, United States
     * US-NY: New York, United States
     * US-OH: Ohio, United States
     * US-OK: Oklahoma, United States
     * US-OR: Oregon, United States
     * US-PA: Pennsylvania, United States
     * US-PR: Puerto Rico, United States
     * US-RI: Rhode Island, United States
     * US-SC: South Carolina, United States
     * US-SD: South Dakota, United States
     * US-TN: Tennessee, United States
     * US-TX: Texas, United States
     * US-UM: United States Minor Outlying Islands, United States
     * US-UT: Utah, United States
     * US-VA: Virginia, United States
     * US-VI: Virgin Islands, U.S., United States
     * US-VT: Vermont, United States
     * US-WA: Washington, United States
     * US-WI: Wisconsin, United States
     * US-WV: West Virginia, United States
     * US-WY: Wyoming, United States

    UN m49 codes for countries and regions (i.e. both Egypt and Northern Africa)
    See https://unstats.un.org/unsd/methodology/m49/overview/

     * m49_001: Global
     * m49_003: North America. The continent of North America (numerical code 003) comprises Northern America (numerical code 021), Caribbean (numerical code 029), and Central America (numerical code 013).
     * m49_002: Africa
     * m49_015: Northern Africa
     * m49_202: Sub-Saharan Africa
     * m49_019: Americas
     * m49_419: Latin America and the Caribbean
     * m49_021: Northern America
     * m49_142: Asia
     * m49_143: Central Asia
     * m49_030: Eastern Asia
     * m49_035: South-eastern Asia
     * m49_034: Southern Asia
     * m49_145: Western Asia
     * m49_150: Europe
     * m49_151: Eastern Europe
     * m49_154: Northern Europe
     * m49_039: Southern Europe
     * m49_155: Western Europe
     * m49_009: Oceania
     * m49_053: Australia and New Zealand
     * m49_054: Melanesia
     * m49_057: Micronesia
     * m49_061: Polynesia
     * m49_012: Algeria
     * m49_818: Egypt
     * m49_434: Libya
     * m49_504: Morocco
     * m49_729: Sudan
     * m49_788: Tunisia
     * m49_732: Western Sahara
     * m49_086: British Indian Ocean Territory
     * m49_108: Burundi
     * m49_174: Comoros
     * m49_262: Djibouti
     * m49_232: Eritrea
     * m49_231: Ethiopia
     * m49_260: French Southern Territories
     * m49_404: Kenya
     * m49_450: Madagascar
     * m49_454: Malawi
     * m49_480: Mauritius
     * m49_175: Mayotte
     * m49_508: Mozambique
     * m49_638: Réunion
     * m49_646: Rwanda
     * m49_690: Seychelles
     * m49_706: Somalia
     * m49_728: South Sudan
     * m49_800: Uganda
     * m49_834: United Republic of Tanzania
     * m49_894: Zambia
     * m49_716: Zimbabwe
     * m49_024: Angola
     * m49_120: Cameroon
     * m49_140: Central African Republic
     * m49_148: Chad
     * m49_178: Congo
     * m49_180: Democratic Republic of the Congo
     * m49_226: Equatorial Guinea
     * m49_266: Gabon
     * m49_678: Sao Tome and Principe
     * m49_072: Botswana
     * m49_748: Eswatini
     * m49_426: Lesotho
     * m49_516: Namibia
     * m49_710: South Africa
     * m49_204: Benin
     * m49_854: Burkina Faso
     * m49_132: Cabo Verde
     * m49_384: Côte d’Ivoire
     * m49_270: Gambia
     * m49_288: Ghana
     * m49_324: Guinea
     * m49_624: Guinea-Bissau
     * m49_430: Liberia
     * m49_466: Mali
     * m49_478: Mauritania
     * m49_562: Niger
     * m49_566: Nigeria
     * m49_654: Saint Helena
     * m49_686: Senegal
     * m49_694: Sierra Leone
     * m49_768: Togo
     * m49_660: Anguilla
     * m49_028: Antigua and Barbuda
     * m49_533: Aruba
     * m49_044: Bahamas
     * m49_052: Barbados
     * m49_535: Bonaire, Sint Eustatius and Saba
     * m49_092: British Virgin Islands
     * m49_136: Cayman Islands
     * m49_192: Cuba
     * m49_531: Curaçao
     * m49_212: Dominica
     * m49_214: Dominican Republic
     * m49_308: Grenada
     * m49_312: Guadeloupe
     * m49_332: Haiti
     * m49_388: Jamaica
     * m49_474: Martinique
     * m49_500: Montserrat
     * m49_630: Puerto Rico
     * m49_652: Saint Barthélemy
     * m49_659: Saint Kitts and Nevis
     * m49_662: Saint Lucia
     * m49_663: Saint Martin (French Part)
     * m49_670: Saint Vincent and the Grenadines
     * m49_534: Sint Maarten (Dutch part)
     * m49_780: Trinidad and Tobago
     * m49_796: Turks and Caicos Islands
     * m49_850: United States Virgin Islands
     * m49_084: Belize
     * m49_188: Costa Rica
     * m49_222: El Salvador
     * m49_320: Guatemala
     * m49_340: Honduras
     * m49_484: Mexico
     * m49_558: Nicaragua
     * m49_591: Panama
     * m49_032: Argentina
     * m49_068: Bolivia (Plurinational State of)
     * m49_074: Bouvet Island
     * m49_076: Brazil
     * m49_152: Chile
     * m49_170: Colombia
     * m49_218: Ecuador
     * m49_238: Falkland Islands (Malvinas)
     * m49_254: French Guiana
     * m49_328: Guyana
     * m49_600: Paraguay
     * m49_604: Peru
     * m49_239: South Georgia and the South Sandwich Islands
     * m49_740: Suriname
     * m49_858: Uruguay
     * m49_862: Venezuela (Bolivarian Republic of)
     * m49_060: Bermuda
     * m49_124: Canada
     * m49_304: Greenland
     * m49_666: Saint Pierre and Miquelon
     * m49_840: United States of America
     * m49_010: Antarctica
     * m49_398: Kazakhstan
     * m49_417: Kyrgyzstan
     * m49_762: Tajikistan
     * m49_795: Turkmenistan
     * m49_860: Uzbekistan
     * m49_156: China
     * m49_344: China, Hong Kong Special Administrative Region
     * m49_446: China, Macao Special Administrative Region
     * m49_408: Democratic People's Republic of Korea
     * m49_392: Japan
     * m49_496: Mongolia
     * m49_410: Republic of Korea
     * m49_096: Brunei Darussalam
     * m49_116: Cambodia
     * m49_360: Indonesia
     * m49_418: Lao People's Democratic Republic
     * m49_458: Malaysia
     * m49_104: Myanmar
     * m49_608: Philippines
     * m49_702: Singapore
     * m49_764: Thailand
     * m49_626: Timor-Leste
     * m49_704: Viet Nam
     * m49_004: Afghanistan
     * m49_050: Bangladesh
     * m49_064: Bhutan
     * m49_356: India
     * m49_364: Iran (Islamic Republic of)
     * m49_462: Maldives
     * m49_524: Nepal
     * m49_586: Pakistan
     * m49_144: Sri Lanka
     * m49_051: Armenia
     * m49_031: Azerbaijan
     * m49_048: Bahrain
     * m49_196: Cyprus
     * m49_268: Georgia
     * m49_368: Iraq
     * m49_376: Israel
     * m49_400: Jordan
     * m49_414: Kuwait
     * m49_422: Lebanon
     * m49_512: Oman
     * m49_634: Qatar
     * m49_682: Saudi Arabia
     * m49_275: State of Palestine
     * m49_760: Syrian Arab Republic
     * m49_792: Türkiye
     * m49_784: United Arab Emirates
     * m49_887: Yemen
     * m49_112: Belarus
     * m49_100: Bulgaria
     * m49_203: Czechia
     * m49_348: Hungary
     * m49_616: Poland
     * m49_498: Republic of Moldova
     * m49_642: Romania
     * m49_643: Russian Federation
     * m49_703: Slovakia
     * m49_804: Ukraine
     * m49_248: Åland Islands
     * m49_208: Denmark
     * m49_233: Estonia
     * m49_234: Faroe Islands
     * m49_246: Finland
     * m49_831: Guernsey
     * m49_352: Iceland
     * m49_372: Ireland
     * m49_833: Isle of Man
     * m49_832: Jersey
     * m49_428: Latvia
     * m49_440: Lithuania
     * m49_578: Norway
     * m49_744: Svalbard and Jan Mayen Islands
     * m49_752: Sweden
     * m49_826: United Kingdom of Great Britain and Northern Ireland
     * m49_008: Albania
     * m49_020: Andorra
     * m49_070: Bosnia and Herzegovina
     * m49_191: Croatia
     * m49_292: Gibraltar
     * m49_300: Greece
     * m49_336: Holy See
     * m49_380: Italy
     * m49_470: Malta
     * m49_499: Montenegro
     * m49_807: North Macedonia
     * m49_620: Portugal
     * m49_674: San Marino
     * m49_688: Serbia
     * m49_705: Slovenia
     * m49_724: Spain
     * m49_040: Austria
     * m49_056: Belgium
     * m49_250: France
     * m49_276: Germany
     * m49_438: Liechtenstein
     * m49_442: Luxembourg
     * m49_492: Monaco
     * m49_528: Netherlands (Kingdom of the)
     * m49_756: Switzerland
     * m49_036: Australia
     * m49_162: Christmas Island
     * m49_166: Cocos (Keeling) Islands
     * m49_334: Heard Island and McDonald Islands
     * m49_554: New Zealand
     * m49_574: Norfolk Island
     * m49_242: Fiji
     * m49_540: New Caledonia
     * m49_598: Papua New Guinea
     * m49_090: Solomon Islands
     * m49_548: Vanuatu
     * m49_316: Guam
     * m49_296: Kiribati
     * m49_584: Marshall Islands
     * m49_583: Micronesia (Federated States of)
     * m49_520: Nauru
     * m49_580: Northern Mariana Islands
     * m49_585: Palau
     * m49_581: United States Minor Outlying Islands
     * m49_016: American Samoa
     * m49_184: Cook Islands
     * m49_258: French Polynesia
     * m49_570: Niue
     * m49_612: Pitcairn
     * m49_882: Samoa
     * m49_772: Tokelau
     * m49_776: Tonga
     * m49_798: Tuvalu
     * m49_876: Wallis and Futuna Islands

    """

    # Special geography
    EU27 = "EU27"
    NAFTA = "NAFTA"

    # Countries
    AD = "AD"
    AE = "AE"
    AF = "AF"
    AG = "AG"
    AI = "AI"
    AL = "AL"
    AM = "AM"
    AO = "AO"
    AQ = "AQ"
    AR = "AR"
    AS = "AS"
    AT = "AT"
    AU = "AU"
    AW = "AW"
    AX = "AX"
    AZ = "AZ"
    BA = "BA"
    BB = "BB"
    BD = "BD"
    BE = "BE"
    BF = "BF"
    BG = "BG"
    BH = "BH"
    BI = "BI"
    BJ = "BJ"
    BL = "BL"
    BM = "BM"
    BN = "BN"
    BO = "BO"
    BQ = "BQ"
    BR = "BR"
    BS = "BS"
    BT = "BT"
    BV = "BV"
    BW = "BW"
    BY = "BY"
    BZ = "BZ"
    CA = "CA"
    CC = "CC"
    CD = "CD"
    CF = "CF"
    CG = "CG"
    CH = "CH"
    CI = "CI"
    CK = "CK"
    CL = "CL"
    CM = "CM"
    CN = "CN"
    CO = "CO"
    CR = "CR"
    CU = "CU"
    CV = "CV"
    CW = "CW"
    CX = "CX"
    CY = "CY"
    CZ = "CZ"
    DE = "DE"
    DJ = "DJ"
    DK = "DK"
    DM = "DM"
    DO = "DO"
    DZ = "DZ"
    EC = "EC"
    EE = "EE"
    EG = "EG"
    EH = "EH"
    ER = "ER"
    ES = "ES"
    ET = "ET"
    FI = "FI"
    FJ = "FJ"
    FK = "FK"
    FM = "FM"
    FO = "FO"
    FR = "FR"
    GA = "GA"
    GB = "GB"
    GD = "GD"
    GE = "GE"
    GF = "GF"
    GG = "GG"
    GH = "GH"
    GI = "GI"
    GL = "GL"
    GM = "GM"
    GN = "GN"
    GP = "GP"
    GQ = "GQ"
    GR = "GR"
    GS = "GS"
    GT = "GT"
    GU = "GU"
    GW = "GW"
    GY = "GY"
    HK = "HK"
    HM = "HM"
    HN = "HN"
    HR = "HR"
    HT = "HT"
    HU = "HU"
    ID = "ID"
    IE = "IE"
    IL = "IL"
    IM = "IM"
    IN = "IN"
    IO = "IO"
    IQ = "IQ"
    IR = "IR"
    IS = "IS"
    IT = "IT"
    JE = "JE"
    JM = "JM"
    JO = "JO"
    JP = "JP"
    KE = "KE"
    KG = "KG"
    KH = "KH"
    KI = "KI"
    KM = "KM"
    KN = "KN"
    KP = "KP"
    KR = "KR"
    KW = "KW"
    KY = "KY"
    KZ = "KZ"
    LA = "LA"
    LB = "LB"
    LC = "LC"
    LI = "LI"
    LK = "LK"
    LR = "LR"
    LS = "LS"
    LT = "LT"
    LU = "LU"
    LV = "LV"
    LY = "LY"
    MA = "MA"
    MC = "MC"
    MD = "MD"
    ME = "ME"
    MF = "MF"
    MG = "MG"
    MH = "MH"
    MK = "MK"
    ML = "ML"
    MM = "MM"
    MN = "MN"
    MO = "MO"
    MP = "MP"
    MQ = "MQ"
    MR = "MR"
    MS = "MS"
    MT = "MT"
    MU = "MU"
    MV = "MV"
    MW = "MW"
    MX = "MX"
    MY = "MY"
    MZ = "MZ"
    NA = "NA"
    NC = "NC"
    NE = "NE"
    NF = "NF"
    NG = "NG"
    NI = "NI"
    NL = "NL"
    NO = "NO"
    NP = "NP"
    NR = "NR"
    NU = "NU"
    NZ = "NZ"
    OM = "OM"
    PA = "PA"
    PE = "PE"
    PF = "PF"
    PG = "PG"
    PH = "PH"
    PK = "PK"
    PL = "PL"
    PM = "PM"
    PN = "PN"
    PR = "PR"
    PS = "PS"
    PT = "PT"
    PW = "PW"
    PY = "PY"
    QA = "QA"
    RE = "RE"
    RO = "RO"
    RS = "RS"
    RU = "RU"
    RW = "RW"
    SA = "SA"
    SB = "SB"
    SC = "SC"
    SD = "SD"
    SE = "SE"
    SG = "SG"
    SH = "SH"
    SI = "SI"
    SJ = "SJ"
    SK = "SK"
    SL = "SL"
    SM = "SM"
    SN = "SN"
    SO = "SO"
    SR = "SR"
    SS = "SS"
    ST = "ST"
    SV = "SV"
    SX = "SX"
    SY = "SY"
    SZ = "SZ"
    TC = "TC"
    TD = "TD"
    TF = "TF"
    TG = "TG"
    TH = "TH"
    TJ = "TJ"
    TK = "TK"
    TL = "TL"
    TM = "TM"
    TN = "TN"
    TO = "TO"
    TR = "TR"
    TT = "TT"
    TV = "TV"
    TW = "TW"
    TZ = "TZ"
    UA = "UA"
    UG = "UG"
    UM = "UM"
    US = "US"
    UY = "UY"
    UZ = "UZ"
    VA = "VA"
    VC = "VC"
    VE = "VE"
    VG = "VG"
    VI = "VI"
    VN = "VN"
    VU = "VU"
    WF = "WF"
    WS = "WS"
    YE = "YE"
    YT = "YT"
    ZA = "ZA"
    ZM = "ZM"
    ZW = "ZW"

    # USA and Canada subdivisions
    CA_AB = "CA-AB"
    CA_BC = "CA-BC"
    CA_MB = "CA-MB"
    CA_NB = "CA-NB"
    CA_NL = "CA-NL"
    CA_NS = "CA-NS"
    CA_NT = "CA-NT"
    CA_NU = "CA-NU"
    CA_ON = "CA-ON"
    CA_PE = "CA-PE"
    CA_QC = "CA-QC"
    CA_SK = "CA-SK"
    CA_YT = "CA-YT"
    US_AK = "US-AK"
    US_AL = "US-AL"
    US_AR = "US-AR"
    US_AS = "US-AS"
    US_AZ = "US-AZ"
    US_CA = "US-CA"
    US_CO = "US-CO"
    US_CT = "US-CT"
    US_DC = "US-DC"
    US_DE = "US-DE"
    US_FL = "US-FL"
    US_GA = "US-GA"
    US_GU = "US-GU"
    US_HI = "US-HI"
    US_IA = "US-IA"
    US_ID = "US-ID"
    US_IL = "US-IL"
    US_IN = "US-IN"
    US_KS = "US-KS"
    US_KY = "US-KY"
    US_LA = "US-LA"
    US_MA = "US-MA"
    US_MD = "US-MD"
    US_ME = "US-ME"
    US_MI = "US-MI"
    US_MN = "US-MN"
    US_MO = "US-MO"
    US_MP = "US-MP"
    US_MS = "US-MS"
    US_MT = "US-MT"
    US_NC = "US-NC"
    US_ND = "US-ND"
    US_NE = "US-NE"
    US_NH = "US-NH"
    US_NJ = "US-NJ"
    US_NM = "US-NM"
    US_NV = "US-NV"
    US_NY = "US-NY"
    US_OH = "US-OH"
    US_OK = "US-OK"
    US_OR = "US-OR"
    US_PA = "US-PA"
    US_PR = "US-PR"
    US_RI = "US-RI"
    US_SC = "US-SC"
    US_SD = "US-SD"
    US_TN = "US-TN"
    US_TX = "US-TX"
    US_UM = "US-UM"
    US_UT = "US-UT"
    US_VA = "US-VA"
    US_VI = "US-VI"
    US_VT = "US-VT"
    US_WA = "US-WA"
    US_WI = "US-WI"
    US_WV = "US-WV"
    US_WY = "US-WY"

    # m49 codification
    m49_001 = "001"  # Global
    m49_003 = "003"  # North America. The continent of North America (numerical code 003) comprises Northern America (numerical code 021), Caribbean (numerical code 029), and Central America (numerical code 013).
    m49_002 = "002"  # Africa
    m49_015 = "015"  # Northern Africa
    m49_202 = "202"  # Sub-Saharan Africa
    m49_019 = "019"  # Americas
    m49_419 = "419"  # Latin America and the Caribbean
    m49_021 = "021"  # Northern America
    m49_142 = "142"  # Asia
    m49_143 = "143"  # Central Asia
    m49_030 = "030"  # Eastern Asia
    m49_035 = "035"  # South-eastern Asia
    m49_034 = "034"  # Southern Asia
    m49_145 = "145"  # Western Asia
    m49_150 = "150"  # Europe
    m49_151 = "151"  # Eastern Europe
    m49_154 = "154"  # Northern Europe
    m49_039 = "039"  # Southern Europe
    m49_155 = "155"  # Western Europe
    m49_009 = "009"  # Oceania
    m49_053 = "053"  # Australia and New Zealand
    m49_054 = "054"  # Melanesia
    m49_057 = "057"  # Micronesia
    m49_061 = "061"  # Polynesia
    m49_012 = "012"  # Algeria
    m49_818 = "818"  # Egypt
    m49_434 = "434"  # Libya
    m49_504 = "504"  # Morocco
    m49_729 = "729"  # Sudan
    m49_788 = "788"  # Tunisia
    m49_732 = "732"  # Western Sahara
    m49_086 = "086"  # British Indian Ocean Territory
    m49_108 = "108"  # Burundi
    m49_174 = "174"  # Comoros
    m49_262 = "262"  # Djibouti
    m49_232 = "232"  # Eritrea
    m49_231 = "231"  # Ethiopia
    m49_260 = "260"  # French Southern Territories
    m49_404 = "404"  # Kenya
    m49_450 = "450"  # Madagascar
    m49_454 = "454"  # Malawi
    m49_480 = "480"  # Mauritius
    m49_175 = "175"  # Mayotte
    m49_508 = "508"  # Mozambique
    m49_638 = "638"  # Réunion
    m49_646 = "646"  # Rwanda
    m49_690 = "690"  # Seychelles
    m49_706 = "706"  # Somalia
    m49_728 = "728"  # South Sudan
    m49_800 = "800"  # Uganda
    m49_834 = "834"  # United Republic of Tanzania
    m49_894 = "894"  # Zambia
    m49_716 = "716"  # Zimbabwe
    m49_024 = "024"  # Angola
    m49_120 = "120"  # Cameroon
    m49_140 = "140"  # Central African Republic
    m49_148 = "148"  # Chad
    m49_178 = "178"  # Congo
    m49_180 = "180"  # Democratic Republic of the Congo
    m49_226 = "226"  # Equatorial Guinea
    m49_266 = "266"  # Gabon
    m49_678 = "678"  # Sao Tome and Principe
    m49_072 = "072"  # Botswana
    m49_748 = "748"  # Eswatini
    m49_426 = "426"  # Lesotho
    m49_516 = "516"  # Namibia
    m49_710 = "710"  # South Africa
    m49_204 = "204"  # Benin
    m49_854 = "854"  # Burkina Faso
    m49_132 = "132"  # Cabo Verde
    m49_384 = "384"  # Côte d’Ivoire
    m49_270 = "270"  # Gambia
    m49_288 = "288"  # Ghana
    m49_324 = "324"  # Guinea
    m49_624 = "624"  # Guinea-Bissau
    m49_430 = "430"  # Liberia
    m49_466 = "466"  # Mali
    m49_478 = "478"  # Mauritania
    m49_562 = "562"  # Niger
    m49_566 = "566"  # Nigeria
    m49_654 = "654"  # Saint Helena
    m49_686 = "686"  # Senegal
    m49_694 = "694"  # Sierra Leone
    m49_768 = "768"  # Togo
    m49_660 = "660"  # Anguilla
    m49_028 = "028"  # Antigua and Barbuda
    m49_533 = "533"  # Aruba
    m49_044 = "044"  # Bahamas
    m49_052 = "052"  # Barbados
    m49_535 = "535"  # Bonaire, Sint Eustatius and Saba
    m49_092 = "092"  # British Virgin Islands
    m49_136 = "136"  # Cayman Islands
    m49_192 = "192"  # Cuba
    m49_531 = "531"  # Curaçao
    m49_212 = "212"  # Dominica
    m49_214 = "214"  # Dominican Republic
    m49_308 = "308"  # Grenada
    m49_312 = "312"  # Guadeloupe
    m49_332 = "332"  # Haiti
    m49_388 = "388"  # Jamaica
    m49_474 = "474"  # Martinique
    m49_500 = "500"  # Montserrat
    m49_630 = "630"  # Puerto Rico
    m49_652 = "652"  # Saint Barthélemy
    m49_659 = "659"  # Saint Kitts and Nevis
    m49_662 = "662"  # Saint Lucia
    m49_663 = "663"  # Saint Martin (French Part)
    m49_670 = "670"  # Saint Vincent and the Grenadines
    m49_534 = "534"  # Sint Maarten (Dutch part)
    m49_780 = "780"  # Trinidad and Tobago
    m49_796 = "796"  # Turks and Caicos Islands
    m49_850 = "850"  # United States Virgin Islands
    m49_084 = "084"  # Belize
    m49_188 = "188"  # Costa Rica
    m49_222 = "222"  # El Salvador
    m49_320 = "320"  # Guatemala
    m49_340 = "340"  # Honduras
    m49_484 = "484"  # Mexico
    m49_558 = "558"  # Nicaragua
    m49_591 = "591"  # Panama
    m49_032 = "032"  # Argentina
    m49_068 = "068"  # Bolivia (Plurinational State of)
    m49_074 = "074"  # Bouvet Island
    m49_076 = "076"  # Brazil
    m49_152 = "152"  # Chile
    m49_170 = "170"  # Colombia
    m49_218 = "218"  # Ecuador
    m49_238 = "238"  # Falkland Islands (Malvinas)
    m49_254 = "254"  # French Guiana
    m49_328 = "328"  # Guyana
    m49_600 = "600"  # Paraguay
    m49_604 = "604"  # Peru
    m49_239 = "239"  # South Georgia and the South Sandwich Islands
    m49_740 = "740"  # Suriname
    m49_858 = "858"  # Uruguay
    m49_862 = "862"  # Venezuela (Bolivarian Republic of)
    m49_060 = "060"  # Bermuda
    m49_124 = "124"  # Canada
    m49_304 = "304"  # Greenland
    m49_666 = "666"  # Saint Pierre and Miquelon
    m49_840 = "840"  # United States of America
    m49_010 = "010"  # Antarctica
    m49_398 = "398"  # Kazakhstan
    m49_417 = "417"  # Kyrgyzstan
    m49_762 = "762"  # Tajikistan
    m49_795 = "795"  # Turkmenistan
    m49_860 = "860"  # Uzbekistan
    m49_156 = "156"  # China
    m49_344 = "344"  # China, Hong Kong Special Administrative Region
    m49_446 = "446"  # China, Macao Special Administrative Region
    m49_408 = "408"  # Democratic People's Republic of Korea
    m49_392 = "392"  # Japan
    m49_496 = "496"  # Mongolia
    m49_410 = "410"  # Republic of Korea
    m49_096 = "096"  # Brunei Darussalam
    m49_116 = "116"  # Cambodia
    m49_360 = "360"  # Indonesia
    m49_418 = "418"  # Lao People's Democratic Republic
    m49_458 = "458"  # Malaysia
    m49_104 = "104"  # Myanmar
    m49_608 = "608"  # Philippines
    m49_702 = "702"  # Singapore
    m49_764 = "764"  # Thailand
    m49_626 = "626"  # Timor-Leste
    m49_704 = "704"  # Viet Nam
    m49_004 = "004"  # Afghanistan
    m49_050 = "050"  # Bangladesh
    m49_064 = "064"  # Bhutan
    m49_356 = "356"  # India
    m49_364 = "364"  # Iran (Islamic Republic of)
    m49_462 = "462"  # Maldives
    m49_524 = "524"  # Nepal
    m49_586 = "586"  # Pakistan
    m49_144 = "144"  # Sri Lanka
    m49_051 = "051"  # Armenia
    m49_031 = "031"  # Azerbaijan
    m49_048 = "048"  # Bahrain
    m49_196 = "196"  # Cyprus
    m49_268 = "268"  # Georgia
    m49_368 = "368"  # Iraq
    m49_376 = "376"  # Israel
    m49_400 = "400"  # Jordan
    m49_414 = "414"  # Kuwait
    m49_422 = "422"  # Lebanon
    m49_512 = "512"  # Oman
    m49_634 = "634"  # Qatar
    m49_682 = "682"  # Saudi Arabia
    m49_275 = "275"  # State of Palestine
    m49_760 = "760"  # Syrian Arab Republic
    m49_792 = "792"  # Türkiye
    m49_784 = "784"  # United Arab Emirates
    m49_887 = "887"  # Yemen
    m49_112 = "112"  # Belarus
    m49_100 = "100"  # Bulgaria
    m49_203 = "203"  # Czechia
    m49_348 = "348"  # Hungary
    m49_616 = "616"  # Poland
    m49_498 = "498"  # Republic of Moldova
    m49_642 = "642"  # Romania
    m49_643 = "643"  # Russian Federation
    m49_703 = "703"  # Slovakia
    m49_804 = "804"  # Ukraine
    m49_248 = "248"  # Åland Islands
    m49_208 = "208"  # Denmark
    m49_233 = "233"  # Estonia
    m49_234 = "234"  # Faroe Islands
    m49_246 = "246"  # Finland
    m49_831 = "831"  # Guernsey
    m49_352 = "352"  # Iceland
    m49_372 = "372"  # Ireland
    m49_833 = "833"  # Isle of Man
    m49_832 = "832"  # Jersey
    m49_428 = "428"  # Latvia
    m49_440 = "440"  # Lithuania
    m49_578 = "578"  # Norway
    m49_744 = "744"  # Svalbard and Jan Mayen Islands
    m49_752 = "752"  # Sweden
    m49_826 = "826"  # United Kingdom of Great Britain and Northern Ireland
    m49_008 = "008"  # Albania
    m49_020 = "020"  # Andorra
    m49_070 = "070"  # Bosnia and Herzegovina
    m49_191 = "191"  # Croatia
    m49_292 = "292"  # Gibraltar
    m49_300 = "300"  # Greece
    m49_336 = "336"  # Holy See
    m49_380 = "380"  # Italy
    m49_470 = "470"  # Malta
    m49_499 = "499"  # Montenegro
    m49_807 = "807"  # North Macedonia
    m49_620 = "620"  # Portugal
    m49_674 = "674"  # San Marino
    m49_688 = "688"  # Serbia
    m49_705 = "705"  # Slovenia
    m49_724 = "724"  # Spain
    m49_040 = "040"  # Austria
    m49_056 = "056"  # Belgium
    m49_250 = "250"  # France
    m49_276 = "276"  # Germany
    m49_438 = "438"  # Liechtenstein
    m49_442 = "442"  # Luxembourg
    m49_492 = "492"  # Monaco
    m49_528 = "528"  # Netherlands (Kingdom of the)
    m49_756 = "756"  # Switzerland
    m49_036 = "036"  # Australia
    m49_162 = "162"  # Christmas Island
    m49_166 = "166"  # Cocos (Keeling) Islands
    m49_334 = "334"  # Heard Island and McDonald Islands
    m49_554 = "554"  # New Zealand
    m49_574 = "574"  # Norfolk Island
    m49_242 = "242"  # Fiji
    m49_540 = "540"  # New Caledonia
    m49_598 = "598"  # Papua New Guinea
    m49_090 = "090"  # Solomon Islands
    m49_548 = "548"  # Vanuatu
    m49_316 = "316"  # Guam
    m49_296 = "296"  # Kiribati
    m49_584 = "584"  # Marshall Islands
    m49_583 = "583"  # Micronesia (Federated States of)
    m49_520 = "520"  # Nauru
    m49_580 = "580"  # Northern Mariana Islands
    m49_585 = "585"  # Palau
    m49_581 = "581"  # United States Minor Outlying Islands
    m49_016 = "016"  # American Samoa
    m49_184 = "184"  # Cook Islands
    m49_258 = "258"  # French Polynesia
    m49_570 = "570"  # Niue
    m49_612 = "612"  # Pitcairn
    m49_882 = "882"  # Samoa
    m49_772 = "772"  # Tokelau
    m49_776 = "776"  # Tonga
    m49_798 = "798"  # Tuvalu
    m49_876 = "876"  # Wallis and Futuna Islands


class Country(StrEnum):
    """
    Geography definitions.

    Countries, see https://en.wikipedia.org/wiki/ISO_3166-1

     * AD: Andorra
     * AE: United Arab Emirates
     * AF: Afghanistan
     * AG: Antigua and Barbuda
     * AI: Anguilla
     * AL: Albania
     * AM: Armenia
     * AO: Angola
     * AQ: Antarctica
     * AR: Argentina
     * AS: American Samoa
     * AT: Austria
     * AU: Australia
     * AW: Aruba
     * AX: Åland Islands
     * AZ: Azerbaijan
     * BA: Bosnia and Herzegovina
     * BB: Barbados
     * BD: Bangladesh
     * BE: Belgium
     * BF: Burkina Faso
     * BG: Bulgaria
     * BH: Bahrain
     * BI: Burundi
     * BJ: Benin
     * BL: Saint Barthélemy
     * BM: Bermuda
     * BN: Brunei Darussalam
     * BO: Bolivia, Plurinational State of
     * BQ: Bonaire, Sint Eustatius and Saba
     * BR: Brazil
     * BS: Bahamas
     * BT: Bhutan
     * BV: Bouvet Island
     * BW: Botswana
     * BY: Belarus
     * BZ: Belize
     * CA: Canada
     * CC: Cocos (Keeling) Islands
     * CD: Congo, The Democratic Republic of the
     * CF: Central African Republic
     * CG: Congo
     * CH: Switzerland
     * CI: Côte d'Ivoire
     * CK: Cook Islands
     * CL: Chile
     * CM: Cameroon
     * CN: China
     * CO: Colombia
     * CR: Costa Rica
     * CU: Cuba
     * CV: Cabo Verde
     * CW: Curaçao
     * CX: Christmas Island
     * CY: Cyprus
     * CZ: Czechia
     * DE: Germany
     * DJ: Djibouti
     * DK: Denmark
     * DM: Dominica
     * DO: Dominican Republic
     * DZ: Algeria
     * EC: Ecuador
     * EE: Estonia
     * EG: Egypt
     * EH: Western Sahara
     * ER: Eritrea
     * ES: Spain
     * ET: Ethiopia
     * FI: Finland
     * FJ: Fiji
     * FK: Falkland Islands (Malvinas)
     * FM: Micronesia, Federated States of
     * FO: Faroe Islands
     * FR: France
     * GA: Gabon
     * GB: United Kingdom
     * GD: Grenada
     * GE: Georgia
     * GF: French Guiana
     * GG: Guernsey
     * GH: Ghana
     * GI: Gibraltar
     * GL: Greenland
     * GM: Gambia
     * GN: Guinea
     * GP: Guadeloupe
     * GQ: Equatorial Guinea
     * GR: Greece
     * GS: South Georgia and the South Sandwich Islands
     * GT: Guatemala
     * GU: Guam
     * GW: Guinea-Bissau
     * GY: Guyana
     * HK: Hong Kong
     * HM: Heard Island and McDonald Islands
     * HN: Honduras
     * HR: Croatia
     * HT: Haiti
     * HU: Hungary
     * ID: Indonesia
     * IE: Ireland
     * IL: Israel
     * IM: Isle of Man
     * IN: India
     * IO: British Indian Ocean Territory
     * IQ: Iraq
     * IR: Iran, Islamic Republic of
     * IS: Iceland
     * IT: Italy
     * JE: Jersey
     * JM: Jamaica
     * JO: Jordan
     * JP: Japan
     * KE: Kenya
     * KG: Kyrgyzstan
     * KH: Cambodia
     * KI: Kiribati
     * KM: Comoros
     * KN: Saint Kitts and Nevis
     * KP: Korea, Democratic People's Republic of
     * KR: Korea, Republic of
     * KW: Kuwait
     * KY: Cayman Islands
     * KZ: Kazakhstan
     * LA: Lao People's Democratic Republic
     * LB: Lebanon
     * LC: Saint Lucia
     * LI: Liechtenstein
     * LK: Sri Lanka
     * LR: Liberia
     * LS: Lesotho
     * LT: Lithuania
     * LU: Luxembourg
     * LV: Latvia
     * LY: Libya
     * MA: Morocco
     * MC: Monaco
     * MD: Moldova, Republic of
     * ME: Montenegro
     * MF: Saint Martin (French part)
     * MG: Madagascar
     * MH: Marshall Islands
     * MK: North Macedonia
     * ML: Mali
     * MM: Myanmar
     * MN: Mongolia
     * MO: Macao
     * MP: Northern Mariana Islands
     * MQ: Martinique
     * MR: Mauritania
     * MS: Montserrat
     * MT: Malta
     * MU: Mauritius
     * MV: Maldives
     * MW: Malawi
     * MX: Mexico
     * MY: Malaysia
     * MZ: Mozambique
     * NA: Namibia
     * NC: New Caledonia
     * NE: Niger
     * NF: Norfolk Island
     * NG: Nigeria
     * NI: Nicaragua
     * NL: Netherlands
     * NO: Norway
     * NP: Nepal
     * NR: Nauru
     * NU: Niue
     * NZ: New Zealand
     * OM: Oman
     * PA: Panama
     * PE: Peru
     * PF: French Polynesia
     * PG: Papua New Guinea
     * PH: Philippines
     * PK: Pakistan
     * PL: Poland
     * PM: Saint Pierre and Miquelon
     * PN: Pitcairn
     * PR: Puerto Rico
     * PS: Palestine, State of
     * PT: Portugal
     * PW: Palau
     * PY: Paraguay
     * QA: Qatar
     * RE: Réunion
     * RO: Romania
     * RS: Serbia
     * RU: Russian Federation
     * RW: Rwanda
     * SA: Saudi Arabia
     * SB: Solomon Islands
     * SC: Seychelles
     * SD: Sudan
     * SE: Sweden
     * SG: Singapore
     * SH: Saint Helena, Ascension and Tristan da Cunha
     * SI: Slovenia
     * SJ: Svalbard and Jan Mayen
     * SK: Slovakia
     * SL: Sierra Leone
     * SM: San Marino
     * SN: Senegal
     * SO: Somalia
     * SR: Suriname
     * SS: South Sudan
     * ST: Sao Tome and Principe
     * SV: El Salvador
     * SX: Sint Maarten (Dutch part)
     * SY: Syrian Arab Republic
     * SZ: Eswatini
     * TC: Turks and Caicos Islands
     * TD: Chad
     * TF: French Southern Territories
     * TG: Togo
     * TH: Thailand
     * TJ: Tajikistan
     * TK: Tokelau
     * TL: Timor-Leste
     * TM: Turkmenistan
     * TN: Tunisia
     * TO: Tonga
     * TR: Türkiye
     * TT: Trinidad and Tobago
     * TV: Tuvalu
     * TW: Taiwan, Province of China
     * TZ: Tanzania, United Republic of
     * UA: Ukraine
     * UG: Uganda
     * UM: United States Minor Outlying Islands
     * US: United States
     * UY: Uruguay
     * UZ: Uzbekistan
     * VA: Holy See (Vatican City State)
     * VC: Saint Vincent and the Grenadines
     * VE: Venezuela, Bolivarian Republic of
     * VG: Virgin Islands, British
     * VI: Virgin Islands, U.S.
     * VN: Viet Nam
     * VU: Vanuatu
     * WF: Wallis and Futuna
     * WS: Samoa
     * YE: Yemen
     * YT: Mayotte
     * ZA: South Africa
     * ZM: Zambia
     * ZW: Zimbabwe
    """

    # Countries
    AD = "AD"
    AE = "AE"
    AF = "AF"
    AG = "AG"
    AI = "AI"
    AL = "AL"
    AM = "AM"
    AO = "AO"
    AQ = "AQ"
    AR = "AR"
    AS = "AS"
    AT = "AT"
    AU = "AU"
    AW = "AW"
    AX = "AX"
    AZ = "AZ"
    BA = "BA"
    BB = "BB"
    BD = "BD"
    BE = "BE"
    BF = "BF"
    BG = "BG"
    BH = "BH"
    BI = "BI"
    BJ = "BJ"
    BL = "BL"
    BM = "BM"
    BN = "BN"
    BO = "BO"
    BQ = "BQ"
    BR = "BR"
    BS = "BS"
    BT = "BT"
    BV = "BV"
    BW = "BW"
    BY = "BY"
    BZ = "BZ"
    CA = "CA"
    CC = "CC"
    CD = "CD"
    CF = "CF"
    CG = "CG"
    CH = "CH"
    CI = "CI"
    CK = "CK"
    CL = "CL"
    CM = "CM"
    CN = "CN"
    CO = "CO"
    CR = "CR"
    CU = "CU"
    CV = "CV"
    CW = "CW"
    CX = "CX"
    CY = "CY"
    CZ = "CZ"
    DE = "DE"
    DJ = "DJ"
    DK = "DK"
    DM = "DM"
    DO = "DO"
    DZ = "DZ"
    EC = "EC"
    EE = "EE"
    EG = "EG"
    EH = "EH"
    ER = "ER"
    ES = "ES"
    ET = "ET"
    FI = "FI"
    FJ = "FJ"
    FK = "FK"
    FM = "FM"
    FO = "FO"
    FR = "FR"
    GA = "GA"
    GB = "GB"
    GD = "GD"
    GE = "GE"
    GF = "GF"
    GG = "GG"
    GH = "GH"
    GI = "GI"
    GL = "GL"
    GM = "GM"
    GN = "GN"
    GP = "GP"
    GQ = "GQ"
    GR = "GR"
    GS = "GS"
    GT = "GT"
    GU = "GU"
    GW = "GW"
    GY = "GY"
    HK = "HK"
    HM = "HM"
    HN = "HN"
    HR = "HR"
    HT = "HT"
    HU = "HU"
    ID = "ID"
    IE = "IE"
    IL = "IL"
    IM = "IM"
    IN = "IN"
    IO = "IO"
    IQ = "IQ"
    IR = "IR"
    IS = "IS"
    IT = "IT"
    JE = "JE"
    JM = "JM"
    JO = "JO"
    JP = "JP"
    KE = "KE"
    KG = "KG"
    KH = "KH"
    KI = "KI"
    KM = "KM"
    KN = "KN"
    KP = "KP"
    KR = "KR"
    KW = "KW"
    KY = "KY"
    KZ = "KZ"
    LA = "LA"
    LB = "LB"
    LC = "LC"
    LI = "LI"
    LK = "LK"
    LR = "LR"
    LS = "LS"
    LT = "LT"
    LU = "LU"
    LV = "LV"
    LY = "LY"
    MA = "MA"
    MC = "MC"
    MD = "MD"
    ME = "ME"
    MF = "MF"
    MG = "MG"
    MH = "MH"
    MK = "MK"
    ML = "ML"
    MM = "MM"
    MN = "MN"
    MO = "MO"
    MP = "MP"
    MQ = "MQ"
    MR = "MR"
    MS = "MS"
    MT = "MT"
    MU = "MU"
    MV = "MV"
    MW = "MW"
    MX = "MX"
    MY = "MY"
    MZ = "MZ"
    NA = "NA"
    NC = "NC"
    NE = "NE"
    NF = "NF"
    NG = "NG"
    NI = "NI"
    NL = "NL"
    NO = "NO"
    NP = "NP"
    NR = "NR"
    NU = "NU"
    NZ = "NZ"
    OM = "OM"
    PA = "PA"
    PE = "PE"
    PF = "PF"
    PG = "PG"
    PH = "PH"
    PK = "PK"
    PL = "PL"
    PM = "PM"
    PN = "PN"
    PR = "PR"
    PS = "PS"
    PT = "PT"
    PW = "PW"
    PY = "PY"
    QA = "QA"
    RE = "RE"
    RO = "RO"
    RS = "RS"
    RU = "RU"
    RW = "RW"
    SA = "SA"
    SB = "SB"
    SC = "SC"
    SD = "SD"
    SE = "SE"
    SG = "SG"
    SH = "SH"
    SI = "SI"
    SJ = "SJ"
    SK = "SK"
    SL = "SL"
    SM = "SM"
    SN = "SN"
    SO = "SO"
    SR = "SR"
    SS = "SS"
    ST = "ST"
    SV = "SV"
    SX = "SX"
    SY = "SY"
    SZ = "SZ"
    TC = "TC"
    TD = "TD"
    TF = "TF"
    TG = "TG"
    TH = "TH"
    TJ = "TJ"
    TK = "TK"
    TL = "TL"
    TM = "TM"
    TN = "TN"
    TO = "TO"
    TR = "TR"
    TT = "TT"
    TV = "TV"
    TW = "TW"
    TZ = "TZ"
    UA = "UA"
    UG = "UG"
    UM = "UM"
    US = "US"
    UY = "UY"
    UZ = "UZ"
    VA = "VA"
    VC = "VC"
    VE = "VE"
    VG = "VG"
    VI = "VI"
    VN = "VN"
    VU = "VU"
    WF = "WF"
    WS = "WS"
    YE = "YE"
    YT = "YT"
    ZA = "ZA"
    ZM = "ZM"
    ZW = "ZW"



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
from openepd.api.base_sync_client import BaseApiMethodGroup
from openepd.model.pcr import Pcr, PcrRef


class PcrApi(BaseApiMethodGroup):
    """API methods for EPDs."""

    def get_by_openxpd_uuid(self, uuid: str) -> Pcr:
        """
        Get PCR by Open xPD UUID.

        :param uuid: Open xPD UUID
        :return: PCR
        :raise ObjectNotFound: if PCR not found
        :raise ValidationError: if openxpd_uuid is invalid
        """
        content = self._client.do_request("get", f"/pcrs/{uuid}").json()
        return Pcr.parse_obj(content)

    def create(self, pcr: Pcr) -> PcrRef:
        """
        Create a new PCR.

        :param pcr: PCR to create
        :return: reference to the created PCR
        :raise ValidationError: if given object PCR is invalid
        """
        pcr_ref_obj = self._client.do_request("post", "/pcrs", json=pcr.to_serializable()).json()
        return PcrRef.parse_obj(pcr_ref_obj)



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
from openepd.compat.pydantic import pyd
from openepd.model.specs.base import BaseOpenEpdHierarchicalSpec
from openepd.model.specs.enums import AllFabrication, AllTimberSpecies
from openepd.model.specs.singular.common import HasForestPracticesCertifiers


class WoodJoistsV1(BaseOpenEpdHierarchicalSpec, HasForestPracticesCertifiers):
    """
    Wood joists.

    Prefabricated I-shaped engineered wood structural members made primarily from one or more types of wood. Includes
    products made with metallic webbing. Excludes products where the wood is merely decorative.
    """

    _EXT_VERSION = "1.0"

    # Own fields:
    timber_species: AllTimberSpecies | None = pyd.Field(default=None, description="", example="Alaska Cedar")
    fabrication: AllFabrication | None = pyd.Field(default=None, description="", example="LVL")
    weather_exposed: bool | None = pyd.Field(default=None, description="", example=True)
    fire_retardant: bool | None = pyd.Field(default=None, description="", example=True)
    decay_resistant: bool | None = pyd.Field(default=None, description="", example=True)



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
import csv
import io
from os import PathLike
from typing import IO, Callable, Iterator, Sequence, Type, cast
import zipfile

from openepd.bundle.base import AssetFilter, AssetRef, BaseBundleReader
from openepd.bundle.model import AssetInfo, AssetType, BundleManifest
from openepd.model.base import TOpenEpdObject


class DefaultBundleReader(BaseBundleReader):
    """Default bundle reader implementation. Reads the bundle from a ZIP file."""

    def __init__(self, bundle_file: PathLike | IO[bytes] | str):
        self._bundle_archive = zipfile.ZipFile(bundle_file, mode="r")
        try:
            with self._bundle_archive.open("manifest", "r") as manifest_stream:
                self.__manifest = BundleManifest.parse_raw(manifest_stream.read())
        except Exception as e:
            raise ValueError("The bundle file is not valid. Manifest reading error: " + str(e)) from e
        try:
            self.__check_toc()
        except Exception as e:
            raise ValueError("The bundle file is not valid. TOC reading error: " + str(e)) from e

    def close(self):
        """Close the reader."""
        self._bundle_archive.close()

    def get_manifest(self) -> BundleManifest:
        """Get the manifest of the bundle. Manifest object is immutable."""
        return self.__manifest.copy(deep=True)

    def __create_asset_filter(
        self,
        asset_type: AssetType | str | None = None,
        name: str | None = None,
        parent_ref: AssetRef | None = None,
        ref_type: str | None = None,
        is_translated: bool | None = None,
    ) -> AssetFilter:
        def _filter(a: AssetInfo):
            if asset_type is not None and a.type != asset_type:
                return False
            if name is not None and a.name != name:
                return False
            if parent_ref is not None and a.rel_asset != parent_ref:
                return False
            if ref_type is not None and a.rel_type != ref_type:
                return False
            if is_translated is not None and a.lang is not None and "translated" in a.lang:
                return True
            return True

        return _filter

    def __preprocess_csv_dict(self, input_dict: dict[str, str | None]) -> dict[str, str | None]:
        default_to_none_fields = ("rel_type", "rel_asset", "lang", "content_type", "custom_type", "custom_data")
        for x in default_to_none_fields:
            if input_dict[x] == "":
                input_dict[x] = None
        return input_dict

    def assets_iter(self) -> Iterator[AssetInfo]:
        """Iterate over all assets in the bundle."""
        with self._bundle_archive.open("toc", "r") as toc_stream:
            toc_reader = csv.DictReader(io.TextIOWrapper(toc_stream, encoding="utf-8"), dialect="toc")
            for x in toc_reader:
                yield AssetInfo.parse_obj(self.__preprocess_csv_dict(x))

    def __check_toc(self):
        with self._bundle_archive.open("toc", "r") as toc_stream:
            toc_reader = csv.DictReader(io.TextIOWrapper(toc_stream, encoding="utf-8"), dialect="toc")
            if not toc_reader.fieldnames or len(toc_reader.fieldnames) < len(self._TOC_FIELDS):
                raise ValueError("The bundle file is not valid. TOC reading error: wrong number of fields")

    def root_assets_iter(
        self,
        filter_or_type: AssetFilter | str | AssetType | None = None,
        name: str | None = None,
        parent_ref: AssetRef | None = None,
        ref_type: str | None = None,
        is_translated: bool | None = None,
    ) -> Iterator[AssetInfo]:
        """Iterate over all root assets in the bundle."""
        _filter: AssetFilter
        if isinstance(filter_or_type, Callable):  # type: ignore
            _filter = filter_or_type  # type: ignore
        else:
            _filter = self.__create_asset_filter(
                asset_type=cast(str, filter_or_type),
                name=name,
                parent_ref=parent_ref,
                ref_type=ref_type,
                is_translated=is_translated,
            )

        for x in self.assets_iter():
            if _filter(x) and x.rel_asset is None:
                yield x

    def get_relative_assets_iter(
        self, asset: AssetRef, rel_type: str | Sequence[str] | None = None
    ) -> Iterator[AssetInfo]:
        """Iterate over all assets that are relative to the given asset."""
        if rel_type is not None and isinstance(rel_type, str):
            rel_type = [rel_type]
        asset_ref = self._asset_ref_to_str(asset)
        for x in self.assets_iter():
            if x.rel_asset == asset_ref:
                if rel_type is None or x.rel_type in rel_type:
                    yield x

    def get_asset_by_ref(self, asset_ref: AssetRef) -> AssetInfo | None:
        """Get the asset by its reference."""
        if isinstance(asset_ref, AssetInfo):
            return asset_ref
        asset_ref = self._asset_ref_to_str(asset_ref)
        for x in self.assets_iter():
            if x.ref == asset_ref:
                return x
        return None

    def read_blob_asset(self, asset_ref: AssetRef) -> IO[bytes]:
        """Read the blob asset."""
        asset = self.get_asset_by_ref(asset_ref)
        if asset is None:
            raise ValueError("Asset not found")
        return self._bundle_archive.open(asset.ref, "r")

    def read_object_asset(self, obj_class: Type[TOpenEpdObject], asset_ref: AssetRef) -> TOpenEpdObject:
        """Read the object asset."""
        asset = self.get_asset_by_ref(asset_ref)
        if asset is None:
            raise ValueError("Asset not found")
        if obj_class.get_asset_type() is None:
            raise ValueError(f"Target object {obj_class.__name__} is not supported asset")
        if asset.type != obj_class.get_asset_type():
            raise ValueError(f"Asset type mismatch. Expected {obj_class.get_asset_type()}, got {asset.type}")
        with self._bundle_archive.open(asset.ref, "r") as asset_stream:
            return obj_class.parse_raw(asset_stream.read())



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
from typing import Literal, overload

from requests import Response

from openepd.api.base_sync_client import BaseApiMethodGroup
from openepd.api.common import StreamingListResponse
from openepd.api.epd.dto import EpdSearchResponse, EpdStatisticsResponse, StatisticsDto
from openepd.api.utils import encode_path_param, remove_none_id_fields
from openepd.model.epd import Epd


class EpdApi(BaseApiMethodGroup):
    """API methods for EPDs."""

    def get_by_openxpd_uuid(self, uuid: str) -> Epd:
        """
        Get EPD by OpenEPD UUID.

        :param uuid: OpenEPD UUID
        :return: EPD
        :raise ObjectNotFound: if EPD is not found
        """
        content = self._client.do_request("get", f"/epds/{uuid}").json()
        return Epd.parse_obj(content)

    def find_raw(self, omf: str, page_num: int = 1, page_size: int = 10) -> EpdSearchResponse:
        """
        Find EPDs by Open Material Filter(OMF).

        OMF is a query language for searching materials/EPDs, running statistics and so on.
        It can exist in a string form, which is accepted by most of the endpoints.

        :param omf: OMF - open material filter string (see OMF spec).
        :param page_num: page number
        :param page_size: page size
        :return: the list of EPDs
        """
        content = self._client.do_request(
            "get",
            "/v2/epds/search",
            params=dict(
                omf=omf,
                page_number=page_num,
                page_size=page_size,
            ),
        ).json()
        return EpdSearchResponse.parse_obj(content)

    def find(self, omf: str, page_size: int | None = None) -> StreamingListResponse[Epd]:
        """
        Find EPDs by Open Material Filter(OMF).

        OMF is a query language for searching materials/EPDs, running statistics and so on.
        It can exist in a string form, which is accepted by most of the endpoints.

        :param omf: OMF - open material filter string (see OMF spec).
        :param page_size: page size, None for default
        :return: streaming list of EPDs
        """

        def _get_page(p_num: int, p_size: int) -> EpdSearchResponse:
            return self.find_raw(omf, page_num=p_num, page_size=p_size)

        return StreamingListResponse[Epd](_get_page, page_size=page_size)

    def get_statistics_raw(self, omf: str) -> EpdStatisticsResponse:
        """
        Get statistics for a given search query.

        Statistics contains aggregated parameters such as percentiles distributions of GWP and other parameters.
        Please note - in addition to product EPDs statistics might include industry-wide EPDs and even generic
        estimates to provide a more complete picture where there is not enough product EPDs.

        :param omf: OMF - open material filter string (see OMF spec).
        :return: statistics wrapped in OpenEpdApiResponse
        """
        content = self._client.do_request("get", "/v2/epds/statistics", params=dict(omf=omf)).json()
        return EpdStatisticsResponse.parse_obj(content)

    def get_statistics(self, omf: str) -> StatisticsDto:
        """
        Get statistics for a given search query.

        Statistics contains aggregated parameters such as percentiles distributions of GWP and other parameters.
        Please note - in addition to product EPDs statistics might include industry-wide EPDs and even generic
        estimates to provide a more complete picture where there is not enough product EPDs.

        :param omf: OMF - open material filter string (see OMF spec).
        :return: statistics wrapped in OpenEpdApiResponse
        """
        return self.get_statistics_raw(omf).payload

    @overload
    def post_with_refs(self, epd: Epd, with_response: Literal[True]) -> tuple[Epd, Response]: ...

    @overload
    def post_with_refs(self, epd: Epd, with_response: Literal[False] = False) -> Epd: ...

    def post_with_refs(self, epd: Epd, with_response: bool = False) -> Epd | tuple[Epd, Response]:
        """
        Post an EPD with references.

        :param epd: EPD
        :param with_response: return the response object togather with the EPD
        :param exclude_defaults: If True, fields with default values are excluded from the payload
        :return: EPD or EPD with HTTP Response object depending on parameter
        """
        epd_data = epd.to_serializable(exclude_unset=True, by_alias=True)
        # Remove 'id' fields with None values, as 'id' cannot be None
        epd_data = remove_none_id_fields(epd_data)
        response = self._client.do_request(
            "patch",
            "/epds/post-with-refs",
            json=epd_data,
        )
        content = response.json()
        if with_response:
            return Epd.parse_obj(content), response
        return Epd.parse_obj(content)

    @overload
    def create(self, epd: Epd, with_response: Literal[True]) -> tuple[Epd, Response]: ...

    @overload
    def create(self, epd: Epd, with_response: Literal[False] = False) -> Epd: ...

    def create(self, epd: Epd, with_response: bool = False) -> Epd | tuple[Epd, Response]:
        """
        Create an EPD.

        :param epd: EPD
        :param with_response: return the response object together with the EPD
        :return: EPD or EPD with HTTP Response object depending on parameter
        """
        response = self._client.do_request(
            "post",
            "/epds",
            json=epd.to_serializable(exclude_unset=True, exclude_defaults=True, by_alias=True),
        )
        content = response.json()
        if with_response:
            return Epd.parse_obj(content), response
        return Epd.parse_obj(content)

    @overload
    def edit(self, epd: Epd, with_response: Literal[True]) -> tuple[Epd, Response]: ...

    @overload
    def edit(self, epd: Epd, with_response: Literal[False] = False) -> Epd: ...

    def edit(self, epd: Epd, with_response: bool = False) -> Epd | tuple[Epd, Response]:
        """
        Edit an EPD.

        :param epd: EPD
        :param with_response: return the response object together with the EPD
        :return: EPD or EPD with HTTP Response object depending on parameter
        """
        epd_id = epd.id
        if not epd_id:
            raise ValueError("The EPD ID must be set to edit an EPD.")
        response = self._client.do_request(
            "put",
            f"/epds/{encode_path_param(epd_id)}",
            json=epd.to_serializable(exclude_unset=True, exclude_defaults=True, by_alias=True),
        )
        content = response.json()
        if with_response:
            return Epd.parse_obj(content), response
        return Epd.parse_obj(content)



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
from typing import Literal

from pydantic.v1 import NonNegativeInt

from openepd.compat.pydantic import pyd
from openepd.model.base import BaseDocumentFactory, BaseOpenEpdSchema, OpenEpdDoctypes
from openepd.model.common import WithAltIdsMixin, WithAttachmentsMixin
from openepd.model.declaration import (
    AverageDatasetMixin,
    BaseDeclaration,
    RefBase,
    WithEpdDeveloperMixin,
    WithProgramOperatorMixin,
    WithVerifierMixin,
)
from openepd.model.lcia import WithLciaMixin
from openepd.model.org import Org
from openepd.model.versioning import OpenEpdVersions, Version


class SampleSize(BaseOpenEpdSchema):
    """Sample size."""

    products: NonNegativeInt | None = pyd.Field(
        default=None,
        description="Count of separate products or results that were included in this industry EPD, "
        "and over which the standard deviation was calculated",
    )
    plants: NonNegativeInt | None = pyd.Field(
        default=None, description="Count of unique manufacturing plants that submitted data for this Industry EPD"
    )
    manufacturers: NonNegativeInt | None = pyd.Field(
        default=None, description="Count of unique manufacturing companies that submitted data for this Industry EPD"
    )


class IndustryEpdRef(RefBase, title="Industry EPD (Ref)"):
    """Reference (short) version of Industry average EPD object."""


class IndustryEpdPreviewV0(
    WithAttachmentsMixin,
    AverageDatasetMixin,
    WithEpdDeveloperMixin,
    WithVerifierMixin,
    WithProgramOperatorMixin,
    IndustryEpdRef,
    BaseDeclaration,
    title="Industry EPD (preview)",
):
    """
    Industry EPD Preview object.

    Used in lists and other cases where full LCIA data is not required.
    """

    _FORMAT_VERSION = OpenEpdVersions.Version0.as_str()

    doctype: Literal["openIndustryEpd"] = pyd.Field(
        description='Describes the type and schema of the document. Must always be "openIndustryEpd"',
        default="openIndustryEpd",
    )

    sample_size: SampleSize | None = None

    publishers: list[Org] | None = pyd.Field(description="")
    manufacturers: list[Org] | None = pyd.Field(description="Participating manufacturers")


IndustryEpdPreview = IndustryEpdPreviewV0


class IndustryEpdV0(IndustryEpdPreviewV0, WithLciaMixin, WithAltIdsMixin, title="Industry EPD (Full)"):
    """
    Full Industry EPD object.

    This is considered the most complete valid openEPD object for IndustryEpd. In addition to it, several related
    models are defined, either with fewer fields (to be used in APIs for list requests) or with more relaxed structure
    to support related entities matching.
    """


IndustryEpd = IndustryEpdV0


class IndustryEpdWithDepsV0(IndustryEpdV0, title="Industry EPD (with Dependencies)"):
    """
    Expanded version of the IndustryEpd.

    Contains related entities - orgs - with full fields, to support object matching in implementations.

    For now the implementation matches the above Industry Epd entity, but they will diverge as normal GE would have
    some required fields in Org (like web_domain), and WithDeps would not.

    """


IndustryEpdWithDeps = IndustryEpdWithDepsV0


class IndustryEpdFactory(BaseDocumentFactory[IndustryEpd]):
    """Factory for EPD objects."""

    DOCTYPE_CONSTRAINT = OpenEpdDoctypes.IndustryEpd
    VERSION_MAP: dict[Version, type[IndustryEpd]] = {OpenEpdVersions.Version0: IndustryEpdV0}



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
from typing import Annotated

from openepd.compat.pydantic import pyd
from openepd.model.base import BaseOpenEpdSchema
from openepd.model.specs.base import BaseOpenEpdHierarchicalSpec, BaseOpenEpdSpec, CodegenSpec
from openepd.model.specs.enums import SteelComposition, SteelRebarGrade
from openepd.model.standard import Standard
from openepd.model.validation.numbers import RatioFloat
from openepd.model.validation.quantity import (
    LengthMmStr,
    PressureMPaStr,
    ThermalConductivityStr,
    ThermalExpansionStr,
    validate_quantity_unit_factory,
)


class SteelMakingRoute(BaseOpenEpdSchema):
    """Steel making route."""

    bof: bool | None = pyd.Field(default=None, description="Basic oxygen furnace")
    eaf: bool | None = pyd.Field(default=None, description="Electric arc furnace")
    ohf: bool | None = pyd.Field(default=None, description="Open hearth furnace")


class SteelFabricatedMixin(BaseOpenEpdSpec):
    """Class with fabricated property used in different parts of steel hierarchy."""

    fabricated: bool | None = pyd.Field(default=None, description="", example=True)


class ColdFormedFramingV1(BaseOpenEpdHierarchicalSpec):
    """
    Cold Formed Framing performance specification.

    Cold formed steel elements such as studs and framing, typically made from coil or sheet steel and used
    within walls and ceilings.
    """

    _EXT_VERSION = "1.0"


class DeckingSteelV1(BaseOpenEpdHierarchicalSpec):
    """Corrugated Decking made from cold-formed sheet steel. Often filled with concrete."""

    _EXT_VERSION = "1.0"


class SteelSuspensionAssemblyV1(BaseOpenEpdHierarchicalSpec):
    """Steel suspension assemblies for suspended (e.g. acoustical) ceiling systems."""

    _EXT_VERSION = "1.0"


class HollowSectionsV1(BaseOpenEpdHierarchicalSpec, SteelFabricatedMixin):
    """Hollow cross section steel shape, typically referred to as hollow structural section (HSS)."""

    _EXT_VERSION = "1.0"


class HotRolledSectionsV1(BaseOpenEpdHierarchicalSpec, SteelFabricatedMixin):
    """
    Hot rolled sections performance specification.

    Steel shapes, such as angles, wide flange beams and I-beams, produced using a high temperature
    mill process.
    """

    _EXT_VERSION = "1.0"


class PlateSteelV1(BaseOpenEpdHierarchicalSpec, SteelFabricatedMixin):
    """Plate Steels.

    Flat hot-rolled steel, typically thicker than 'sheet', made by compressing multiple steel
    layers together into one.
    """

    _EXT_VERSION = "1.0"

    # Own fields:


class MetalRailingsV1(BaseOpenEpdHierarchicalSpec):
    """Metal Railings including pipe and tube railings."""

    _EXT_VERSION = "1.0"


class MetalStairsV1(BaseOpenEpdHierarchicalSpec):
    """
    Metal stairs.

    Includes: metal pan stairs, metal floor plate stairs, grating stairs, fire escapes,
    ladders, and walkways/catwalks/ramps/platforms.
    """

    _EXT_VERSION = "1.0"


class MiscMetalFabricationV1(BaseOpenEpdHierarchicalSpec):
    """Prefabricated steel assemblies not included in another category."""

    _EXT_VERSION = "1.0"


class OpenWebMembranesV1(BaseOpenEpdHierarchicalSpec):
    """
    Open web membranes performance specification.

    Lightweight steel truss, typically made of parallel chords and a triangulated web system,
    "proportioned to span between bearing points.
    """

    _EXT_VERSION = "1.0"


class MBQSteelV1(BaseOpenEpdHierarchicalSpec):
    """
    Merchant Bar Quality (MBQ) steel.

    Used as feedstock to steel construction products, but also includes rounds, angles, and light structural shapes.
    """

    _EXT_VERSION = "1.0"


class CoilSteelV1(BaseOpenEpdHierarchicalSpec):
    """
    Sheet or strip steel, sold in rolls.

    Typically, coil steel is cold-formed into light gauge products.
    """

    _EXT_VERSION = "1.0"


class ColdFormedSteelV1(BaseOpenEpdHierarchicalSpec):
    """
    Cold Formed Steel Products.

    All types of cold formed steel products. These products are made from hot-rolled steel coils and
    sheets and are cold formed into products such as studs, decking, panels, and other accessories.
    """

    _EXT_VERSION = "1.0"

    # Nested specs:
    ColdFormedFraming: ColdFormedFramingV1 | None = None
    DeckingSteel: DeckingSteelV1 | None = None
    SteelSuspensionAssembly: SteelSuspensionAssemblyV1 | None = None


class StructuralSteelV1(BaseOpenEpdHierarchicalSpec):
    """
    Structural Steel.

    Hot rolled steel shapes, Hollow Sections, pipes, and similar hot-worked structural steels.
    """

    _EXT_VERSION = "1.0"

    # Own fields:
    modulus_of_elasticity: PressureMPaStr | None = pyd.Field(
        default=None,
        description="Modulus of Elasticity, https://en.wikipedia.org/wiki/Elastic_modulus ",
        example="193 GPa",
    )
    thermal_expansion: ThermalExpansionStr | None = pyd.Field(
        default=None,
        description="Thermal Expansion, https://en.wikipedia.org/wiki/Thermal_expansion",
        example="1.11E-5 / K",
    )
    thermal_conductivity: ThermalConductivityStr | None = pyd.Field(
        default=None,
        description="Thermal Conductivity, https://en.wikipedia.org/wiki/Thermal_conductivity_and_resistivity",
        example="1.45E-5 W / m / K)",
    )

    _steel_thermal_expansion_is_quantity_validator = pyd.validator("thermal_expansion", allow_reuse=True)(
        validate_quantity_unit_factory("1 / K")
    )
    _steel_thermal_conductivity_is_quantity_validator = pyd.validator("thermal_conductivity", allow_reuse=True)(
        validate_quantity_unit_factory("W / (m * K)")
    )

    # Nested specs:
    HollowSections: HollowSectionsV1 | None = None
    HotRolledSections: HotRolledSectionsV1 | None = None
    PlateSteel: PlateSteelV1 | None = None


class PrefabricatedSteelAssembliesV1(BaseOpenEpdHierarchicalSpec):
    """Prefabricated assemblies made primarily of steel."""

    _EXT_VERSION = "1.0"

    # Nested specs:
    MetalRailings: MetalRailingsV1 | None = None
    MetalStairs: MetalStairsV1 | None = None
    MiscMetalFabrication: MiscMetalFabricationV1 | None = None
    OpenWebMembranes: OpenWebMembranesV1 | None = None


class PostTensioningSteelV1(BaseOpenEpdHierarchicalSpec):
    """Steel tensioning cables or tendons for compression of prestressed concrete."""

    _EXT_VERSION = "1.0"


class RebarSteelV1(BaseOpenEpdHierarchicalSpec, SteelFabricatedMixin):
    """Reinforcing bar used together with concrete."""

    _EXT_VERSION = "1.0"

    # Own fields:
    grade: SteelRebarGrade | None = pyd.Field(default=None, example="60 ksi")
    diameter_min: LengthMmStr | None = pyd.Field(default=None, description="Minimal diameter", example="8 mm")
    bending_pin_max: float | None = pyd.Field(default=None, example=2.3)
    ts_ys_ratio_max: float | None = pyd.Field(
        default=None, description="Max ratio of ultimate tensile to yield tensile strength", example=2.3
    )
    epoxy_coated: bool | None = pyd.Field(default=None, example=True)


class WireMeshSteelV1(BaseOpenEpdHierarchicalSpec, SteelFabricatedMixin):
    """Mild steel wire for reinforcement, connections, and meshes."""

    _EXT_VERSION = "1.0"


class OtherSteelV1(BaseOpenEpdHierarchicalSpec):
    """Steel products that do not fit into a defined subcategory."""

    _EXT_VERSION = "1.0"


class SteelV1(BaseOpenEpdHierarchicalSpec):
    """Broad category for construction materials made from steel and its alloys."""

    _EXT_VERSION = "1.1"

    # Own fields:
    yield_tensile_str: PressureMPaStr | None = pyd.Field(
        default=None,
        description="Yield Tensile strength (Mpa) per unit area. Yield strength is the point at which a material "
        "begins to permanently deform or change shape due to applied stress.",
        example="100 MPa",
    )
    bar_elongation: float | None = pyd.Field(
        default=None, description="Increase in length at break, in percent. Typically 10%-20%", example=0.2
    )
    recycled_content: RatioFloat | None = pyd.Field(default=None, description="", example=0.5, ge=0, le=1)
    # todo look how to pass validation down to range fields
    post_consumer_recycled_content: RatioFloat | None = pyd.Field(
        default=None,
        description="Should be a number between zero and the Recycled Content (steel_recycled_content)",
        example=0.5,
        ge=0,
        le=1,
    )
    astm_marking: str | None = pyd.Field(
        default=None, description="The marking to be expected on the product.", example="S4S60"
    )
    euro_marking: str | None = pyd.Field(
        default=None, description="The marking to be expected on the product.", example="S4S60"
    )
    composition: SteelComposition | None = pyd.Field(
        default=None, description="Basic chemical composition", example="Carbon"
    )
    cold_finished: bool | None = pyd.Field(default=None, example=True)
    galvanized: bool | None = pyd.Field(default=None, example=True)
    stainless: bool | None = pyd.Field(default=None, example=True)
    making_route: Annotated[SteelMakingRoute | None, CodegenSpec(override_type=SteelMakingRoute)] = pyd.Field(
        default=None
    )
    astm_standards: list[Standard] | None = pyd.Field(default=None, description="List of ASTM standards")
    sae_standards: list[Standard] | None = pyd.Field(default=None, description="List of SAE standards")
    en_standards: list[Standard] | None = pyd.Field(default=None, description="List of EN standards")

    # Nested specs:
    MBQSteel: MBQSteelV1 | None = None
    CoilSteel: CoilSteelV1 | None = None
    ColdFormedSteel: ColdFormedSteelV1 | None = None
    StructuralSteel: StructuralSteelV1 | None = None
    PrefabricatedSteelAssemblies: PrefabricatedSteelAssembliesV1 | None = None
    PostTensioningSteel: PostTensioningSteelV1 | None = None
    RebarSteel: RebarSteelV1 | None = None
    WireMeshSteel: WireMeshSteelV1 | None = None
    OtherSteel: OtherSteelV1 | None = None



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
from openepd.model.specs.base import BaseOpenEpdHierarchicalSpec


class BlanketFacingV1(BaseOpenEpdHierarchicalSpec):
    """
    Facing materials for insulation products.

    Such as kraft, white vinyl sheeting, or aluminum foil, which can serve as air barrier, vapor barrier, radiant
    barrier, or flame resistive layer.
    """

    _EXT_VERSION = "1.0"


class DoorsHardwareV1(BaseOpenEpdHierarchicalSpec):
    """Door hardware, including automatic and security door hardware."""

    _EXT_VERSION = "1.0"


class FlooringAccessoriesV1(BaseOpenEpdHierarchicalSpec):
    """Products used in flooring, other than the actual flooring product itself."""

    _EXT_VERSION = "1.0"


class MortarV1(BaseOpenEpdHierarchicalSpec):
    """Cementitious paste used to bind building blocks such as stones, bricks, and concrete masonry."""

    _EXT_VERSION = "1.0"


class TileGroutV1(BaseOpenEpdHierarchicalSpec):
    """Water-cement-sand mixture for laying ceramic tile."""

    _EXT_VERSION = "1.0"


class AccessoriesV1(BaseOpenEpdHierarchicalSpec):
    """Materials that are used alongside other materials."""

    _EXT_VERSION = "1.0"

    # Nested specs:
    BlanketFacing: BlanketFacingV1 | None = None
    DoorsHardware: DoorsHardwareV1 | None = None
    FlooringAccessories: FlooringAccessoriesV1 | None = None
    Mortar: MortarV1 | None = None
    TileGrout: TileGroutV1 | None = None



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
__all__ = (
    "HvacVrfControlRangeV1",
    "HvacVrfIndoorRangeV1",
    "HvacVrfOutdoorRangeV1",
    "HvacAirDiffusersRangeV1",
    "HvacAirFiltersRangeV1",
    "HvacAHUsRangeV1",
    "HvacBoilersRangeV1",
    "HvacChillersRangeV1",
    "HvacFansRangeV1",
    "HvacHeatPumpsRangeV1",
    "HvacHeatExRangeV1",
    "HvacPumpsRangeV1",
    "HvacRTUsRangeV1",
    "HvacVrfSystemsRangeV1",
    "HvacDuctsRangeV1",
    "MechanicalRangeV1",
)

# NB! This is a generated code. Do not edit it manually. Please see src/openepd/model/specs/README.md


from openepd.compat.pydantic import pyd
from openepd.model.specs.base import BaseOpenEpdHierarchicalSpec
from openepd.model.specs.enums import (
    AhuAirflowControl,
    AhuZoneControl,
    AirFiltersMediaType,
    AirFiltersMervRating,
    BoilerConfiguration,
    BoilerEquipmentFuelType,
    HeatPumpType,
    HvacDuctMaterial,
    HvacDuctShape,
    HvacDuctType,
    HvacHeatExchangersType,
    MechanicalInstallation,
    MechanicalRefrigerants,
)
from openepd.model.validation.quantity import (
    AmountRangeAirflow,
    AmountRangeAreaPerVolume,
    AmountRangePower,
    AmountRangePressureMpa,
    AmountRangeVolume,
)


class HvacVrfControlRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Controller for adjusting airflow across the VRF system.

    Range version.
    """

    _EXT_VERSION = "1.0"


class HvacVrfIndoorRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Heating and cooling unit located on the inside of a building and supplies air to specific indoor zones.

    Range version.
    """

    _EXT_VERSION = "1.0"

    refrigerants: list[MechanicalRefrigerants] | None = pyd.Field(default=None, description="")
    heating_capacity: AmountRangePower | None = pyd.Field(default=None, description="")
    cooling_capacity: AmountRangePower | None = pyd.Field(default=None, description="")
    airflow_rate: AmountRangeAirflow | None = pyd.Field(default=None, description="")
    air_volume: AmountRangeVolume | None = pyd.Field(default=None, description="")


class HvacVrfOutdoorRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Heating and cooling unit that is on the outside of a building and distributes air to the indoor units.

    Range version.
    """

    _EXT_VERSION = "1.0"

    refrigerants: list[MechanicalRefrigerants] | None = pyd.Field(default=None, description="")
    heating_capacity: AmountRangePower | None = pyd.Field(default=None, description="")
    cooling_capacity: AmountRangePower | None = pyd.Field(default=None, description="")
    airflow_rate: AmountRangeAirflow | None = pyd.Field(default=None, description="")
    air_volume: AmountRangeVolume | None = pyd.Field(default=None, description="")


class HvacAirDiffusersRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Room-side terminals for air distribution. This is different from Terminal Heating & Cooling Units.

    Range version.
    """

    _EXT_VERSION = "1.0"


class HvacAirFiltersRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Device for filtering particles of dust, soot, etc., from the air passing through it.

    Range version.
    """

    _EXT_VERSION = "1.0"

    merv_rating: list[AirFiltersMervRating] | None = pyd.Field(default=None, description="")
    media_type: list[AirFiltersMediaType] | None = pyd.Field(default=None, description="")


class HvacAHUsRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Device which provides healthy, dust free air to buildings with a good energy efficiency.

    Usually a large metal box containing a blower, heating or cooling elements, filter racks or chambers, sound
    attenuators, and dampers.

    Range version.
    """

    _EXT_VERSION = "1.0"

    refrigerants: list[MechanicalRefrigerants] | None = pyd.Field(default=None, description="")
    installation: list[MechanicalInstallation] | None = pyd.Field(default=None, description="")
    airflow_rate: AmountRangeAirflow | None = pyd.Field(default=None, description="")
    air_volume: AmountRangeVolume | None = pyd.Field(default=None, description="")
    cooling_capacity: AmountRangePower | None = pyd.Field(default=None, description="")
    heating_capacity: AmountRangePower | None = pyd.Field(default=None, description="")
    airflow_control: list[AhuAirflowControl] | None = pyd.Field(default=None, description="")
    zone_control: list[AhuZoneControl] | None = pyd.Field(default=None, description="")


class HvacBoilersRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Closed vessel for heating fluid.

    Range version.
    """

    _EXT_VERSION = "1.0"

    flow_rate: AmountRangeAreaPerVolume | None = pyd.Field(default=None, description="")
    heating_capacity: AmountRangePower | None = pyd.Field(default=None, description="")
    configuration: list[BoilerConfiguration] | None = pyd.Field(default=None, description="")
    fuel_type: list[BoilerEquipmentFuelType] | None = pyd.Field(default=None, description="")


class HvacChillersRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Machine that removes heat from a liquid coolant.

    Uses a vapor-compression, adsorption refrigeration, or absorption refrigeration cycles. Incldues centrifugal,
    water-cooled chillers, etc.

    Range version.
    """

    _EXT_VERSION = "1.0"

    refrigerants: list[MechanicalRefrigerants] | None = pyd.Field(default=None, description="")
    installation: list[MechanicalInstallation] | None = pyd.Field(default=None, description="")
    heating_capacity: AmountRangePower | None = pyd.Field(default=None, description="")
    cooling_capacity: AmountRangePower | None = pyd.Field(default=None, description="")
    air_volume: AmountRangeVolume | None = pyd.Field(default=None, description="")
    airflow_rate: AmountRangeAirflow | None = pyd.Field(default=None, description="")


class HvacFansRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Apparatus with rotating blades that creates a current of air for cooling or ventilation.

    Range version.
    """

    _EXT_VERSION = "1.0"


class HvacHeatPumpsRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Device that transfers thermal energy between spaces, including ground and air source heat pumps.

    Range version.
    """

    _EXT_VERSION = "1.0"

    refrigerants: list[MechanicalRefrigerants] | None = pyd.Field(default=None, description="")
    cooling_capacity: AmountRangePower | None = pyd.Field(default=None, description="")
    heating_capacity: AmountRangePower | None = pyd.Field(default=None, description="")
    air_volume: AmountRangeVolume | None = pyd.Field(default=None, description="")
    airflow_rate: AmountRangeAirflow | None = pyd.Field(default=None, description="")
    heat_pumps_type: list[HeatPumpType] | None = pyd.Field(default=None, description="")


class HvacHeatExRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    HVAC heat exchangers.

    Systems that with heat exchange cell which recovers and retains the heat that would otherwise be lost from the
    extracted air.

    Range version.
    """

    _EXT_VERSION = "1.0"

    refrigerants: list[MechanicalRefrigerants] | None = pyd.Field(default=None, description="")
    heat_exchangers_type: list[HvacHeatExchangersType] | None = pyd.Field(default=None, description="")


class HvacPumpsRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Pumps.

    Mechanical device using suction or pressure to raise or move liquids, compress gases, or force air into
    inflatable objects such as tires.

    Range version.
    """

    _EXT_VERSION = "1.0"

    flow_rate: AmountRangeAreaPerVolume | None = pyd.Field(default=None, description="")
    pump_discharge_pressure: AmountRangePressureMpa | None = pyd.Field(default=None, description="")
    pump_horsepower: AmountRangePower | None = pyd.Field(default=None, description="")


class HvacRTUsRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    An air handler designed for outdoor use, typically on roofs.

    Range version.
    """

    _EXT_VERSION = "1.0"


class HvacVrfSystemsRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Variable refrigerant flow (VRF).

    Also known as variable refrigerant volume (VRV), is an HVAC technology that allows for varying degrees of cooling
    in more specific areas.

    Range version.
    """

    _EXT_VERSION = "1.0"

    HvacVrfControl: HvacVrfControlRangeV1 | None = None
    HvacVrfIndoor: HvacVrfIndoorRangeV1 | None = None
    HvacVrfOutdoor: HvacVrfOutdoorRangeV1 | None = None


class HvacDuctsRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Ducts for HVAC systems performance specification.

    Range version.
    """

    _EXT_VERSION = "1.0"

    shape: list[HvacDuctShape] | None = pyd.Field(default=None, description="Hvac duct shape")
    material: list[HvacDuctMaterial] | None = pyd.Field(default=None, description="Hvac duct material")
    type: list[HvacDuctType] | None = pyd.Field(default=None, description="Hvac duct type")


class MechanicalRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Mechanical performance specification.

    Range version.
    """

    _EXT_VERSION = "1.1"

    HvacAirDiffusers: HvacAirDiffusersRangeV1 | None = None
    HvacAirFilters: HvacAirFiltersRangeV1 | None = None
    HvacAHUs: HvacAHUsRangeV1 | None = None
    HvacBoilers: HvacBoilersRangeV1 | None = None
    HvacChillers: HvacChillersRangeV1 | None = None
    HvacFans: HvacFansRangeV1 | None = None
    HvacHeatPumps: HvacHeatPumpsRangeV1 | None = None
    HvacHeatEx: HvacHeatExRangeV1 | None = None
    HvacPumps: HvacPumpsRangeV1 | None = None
    HvacRTUs: HvacRTUsRangeV1 | None = None
    HvacVrfSystems: HvacVrfSystemsRangeV1 | None = None
    HvacDucts: HvacDuctsRangeV1 | None = None



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
__all__ = (
    "DemountablePartitionTrackRangeV1",
    "ChairsRangeV1",
    "CountertopsRangeV1",
    "DemountablePartitionsRangeV1",
    "OtherFurnishingsRangeV1",
    "OpenStorageRangeV1",
    "ClosedStorageRangeV1",
    "RetractableStorageRangeV1",
    "MobileStorageRangeV1",
    "WallMountedShelvingRangeV1",
    "OtherStorageFurnitureRangeV1",
    "StorageFurnitureRangeV1",
    "TablesRangeV1",
    "WorkSurfacesRangeV1",
    "WorkspacesRangeV1",
    "FurnishingsRangeV1",
)

# NB! This is a generated code. Do not edit it manually. Please see src/openepd/model/specs/README.md


from openepd.compat.pydantic import pyd
from openepd.model.common import RangeInt, RangeRatioFloat
from openepd.model.specs.base import BaseOpenEpdHierarchicalSpec
from openepd.model.specs.enums import CountertopMaterial
from openepd.model.validation.quantity import AmountRangeAreaM2, AmountRangeLengthMm, AmountRangeVolume


class DemountablePartitionTrackRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Track for modular partitions.

    Range version.
    """

    _EXT_VERSION = "1.0"


class ChairsRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Chairs.

    Range version.
    """

    _EXT_VERSION = "1.0"


class CountertopsRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Raised, flat, and horizontal surfaces often used in kitchens, bathrooms, and workrooms.

    Range version.
    """

    _EXT_VERSION = "1.0"

    thickness: AmountRangeLengthMm | None = pyd.Field(default=None, description="")
    countertop_material: list[CountertopMaterial] | None = pyd.Field(default=None, description="")


class DemountablePartitionsRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Demountable partitions.

    Range version.
    """

    _EXT_VERSION = "1.0"

    DemountablePartitionTrack: DemountablePartitionTrackRangeV1 | None = None


class OtherFurnishingsRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Other furnishings.

    Range version.
    """

    _EXT_VERSION = "1.0"


class OpenStorageRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Open Storage.

    Open Storage furniture which is static and has no moving parts.

    Range version.
    """

    _EXT_VERSION = "1.0"


class ClosedStorageRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Closed Storage.

    Closed storage furniture making use of doors, sliding and/or hinged parts.

    Range version.
    """

    _EXT_VERSION = "1.0"


class RetractableStorageRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Retractable Storage.

    Storage Furniture with retractable (drawer) elements.

    Range version.
    """

    _EXT_VERSION = "1.0"


class MobileStorageRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Mobile Storage.

    Mobile storage furniture having wheels or casters for movement.

    Range version.
    """

    _EXT_VERSION = "1.0"


class WallMountedShelvingRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Wall Mounted Shelving.

    Storage furniture which requires usage of a vertical structure for attachment and functional support.

    Range version.
    """

    _EXT_VERSION = "1.0"


class OtherStorageFurnitureRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Other Storage Furniture.

    Range version.
    """

    _EXT_VERSION = "1.0"


class StorageFurnitureRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Storage Furniture.

    Range version.
    """

    _EXT_VERSION = "1.1"

    OpenStorage: OpenStorageRangeV1 | None = None
    ClosedStorage: ClosedStorageRangeV1 | None = None
    RetractableStorage: RetractableStorageRangeV1 | None = None
    MobileStorage: MobileStorageRangeV1 | None = None
    WallMountedShelving: WallMountedShelvingRangeV1 | None = None
    OtherStorageFurniture: OtherStorageFurnitureRangeV1 | None = None


class TablesRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Tables.

    Range version.
    """

    _EXT_VERSION = "1.0"


class WorkSurfacesRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Work surfaces.

    Range version.
    """

    _EXT_VERSION = "1.0"


class WorkspacesRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Workspaces.

    Office furniture and furniture systems for performing office work, such as cubicle systems.
    Typically includes component(s) that may fit in other categories.

    Range version.
    """

    _EXT_VERSION = "1.0"


class FurnishingsRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Home and office furnishings.

    Range version.
    """

    _EXT_VERSION = "1.1"

    functional_floor_area_m2: AmountRangeAreaM2 | None = pyd.Field(
        default=None, title="Functional Floor Area", description="The floor area that the product occupies."
    )
    work_surface_area_m2: AmountRangeAreaM2 | None = pyd.Field(
        default=None, title="Work Surface Area", description="The usable work surface area that the product provides."
    )
    functional_storage_volume_m3: AmountRangeVolume | None = pyd.Field(
        default=None, title="Functional Storage Volume", description=""
    )
    functional_seating_capacity: RangeInt | None = pyd.Field(
        default=None,
        title="Functional Seating Capacity",
        description="Intended number of individuals the product seats. This value is used in calculating impact per functional unit.",
    )
    installation_waste_factor: RangeRatioFloat | None = pyd.Field(
        default=None,
        title="Installation Waste Factor",
        description="Typical increase in impacts to account for installation waste.",
    )
    Chairs: ChairsRangeV1 | None = None
    Countertops: CountertopsRangeV1 | None = None
    DemountablePartitions: DemountablePartitionsRangeV1 | None = None
    OtherFurnishings: OtherFurnishingsRangeV1 | None = None
    StorageFurniture: StorageFurnitureRangeV1 | None = None
    Tables: TablesRangeV1 | None = None
    WorkSurfaces: WorkSurfacesRangeV1 | None = None
    Workspaces: WorkspacesRangeV1 | None = None



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
__all__ = ("MechanicalInsulationRangeV1",)

# NB! This is a generated code. Do not edit it manually. Please see src/openepd/model/specs/README.md


from openepd.compat.pydantic import pyd
from openepd.model.common import RangeFloat
from openepd.model.specs.base import BaseOpenEpdHierarchicalSpec
from openepd.model.specs.enums import InsulatingMaterial, InsulationIntendedApplication
from openepd.model.validation.quantity import AmountRangeLengthMm


class MechanicalInsulationRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Insulation products whose primary purpose is for mechanical systems rather than for building envelope.

    Includes HVAC, plumbing, and acoustic insulations.

    Range version.
    """

    _EXT_VERSION = "1.0"

    r_value: RangeFloat | None = pyd.Field(default=None, description="")
    material: list[InsulatingMaterial] | None = pyd.Field(default=None, description="")
    intended_application: list[InsulationIntendedApplication] | None = pyd.Field(default=None, description="")
    thickness_per_declared_unit: AmountRangeLengthMm | None = pyd.Field(default=None, description="")



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
from openepd.compat.pydantic import pyd
from openepd.model.specs.base import BaseOpenEpdHierarchicalSpec
from openepd.model.specs.enums import CountertopMaterial
from openepd.model.validation.numbers import RatioFloat
from openepd.model.validation.quantity import AreaM2Str, LengthMmStr, VolumeStr


class DemountablePartitionTrackV1(BaseOpenEpdHierarchicalSpec):
    """Track for modular partitions."""

    _EXT_VERSION = "1.0"


class ChairsV1(BaseOpenEpdHierarchicalSpec):
    """Chairs."""

    _EXT_VERSION = "1.0"


class CountertopsV1(BaseOpenEpdHierarchicalSpec):
    """Raised, flat, and horizontal surfaces often used in kitchens, bathrooms, and workrooms."""

    _EXT_VERSION = "1.0"

    # Own fields:
    thickness: LengthMmStr | None = pyd.Field(default=None, description="", example="30 mm")
    countertop_material: CountertopMaterial | None = pyd.Field(default=None, description="", example="Stone")


class DemountablePartitionsV1(BaseOpenEpdHierarchicalSpec):
    """Demountable partitions."""

    _EXT_VERSION = "1.0"

    # Nested specs:
    DemountablePartitionTrack: DemountablePartitionTrackV1 | None = None


class OtherFurnishingsV1(BaseOpenEpdHierarchicalSpec):
    """Other furnishings."""

    _EXT_VERSION = "1.0"


class OpenStorageV1(BaseOpenEpdHierarchicalSpec):
    """
    Open Storage.

    Open Storage furniture which is static and has no moving parts.
    """

    _EXT_VERSION = "1.0"


class ClosedStorageV1(BaseOpenEpdHierarchicalSpec):
    """
    Closed Storage.

    Closed storage furniture making use of doors, sliding and/or hinged parts.
    """

    _EXT_VERSION = "1.0"


class RetractableStorageV1(BaseOpenEpdHierarchicalSpec):
    """
    Retractable Storage.

    Storage Furniture with retractable (drawer) elements.
    """

    _EXT_VERSION = "1.0"


class MobileStorageV1(BaseOpenEpdHierarchicalSpec):
    """
    Mobile Storage.

    Mobile storage furniture having wheels or casters for movement.
    """

    _EXT_VERSION = "1.0"


class WallMountedShelvingV1(BaseOpenEpdHierarchicalSpec):
    """
    Wall Mounted Shelving.

    Storage furniture which requires usage of a vertical structure for attachment and functional support.
    """

    _EXT_VERSION = "1.0"


class OtherStorageFurnitureV1(BaseOpenEpdHierarchicalSpec):
    """Other Storage Furniture."""

    _EXT_VERSION = "1.0"


class StorageFurnitureV1(BaseOpenEpdHierarchicalSpec):
    """Storage Furniture."""

    _EXT_VERSION = "1.1"

    # Nested specs:
    OpenStorage: OpenStorageV1 | None = None
    ClosedStorage: ClosedStorageV1 | None = None
    RetractableStorage: RetractableStorageV1 | None = None
    MobileStorage: MobileStorageV1 | None = None
    WallMountedShelving: WallMountedShelvingV1 | None = None
    OtherStorageFurniture: OtherStorageFurnitureV1 | None = None


class TablesV1(BaseOpenEpdHierarchicalSpec):
    """Tables."""

    _EXT_VERSION = "1.0"


class WorkSurfacesV1(BaseOpenEpdHierarchicalSpec):
    """Work surfaces."""

    _EXT_VERSION = "1.0"


class WorkspacesV1(BaseOpenEpdHierarchicalSpec):
    """
    Workspaces.

    Office furniture and furniture systems for performing office work, such as cubicle systems.
    Typically includes component(s) that may fit in other categories.
    """

    _EXT_VERSION = "1.0"


class FurnishingsV1(BaseOpenEpdHierarchicalSpec):
    """Home and office furnishings."""

    _EXT_VERSION = "1.1"

    # Own fields:
    functional_floor_area_m2: AreaM2Str | None = pyd.Field(
        default=None,
        title="Functional Floor Area",
        description="The floor area that the product occupies.",
        example="1 m2",
    )
    work_surface_area_m2: AreaM2Str | None = pyd.Field(
        default=None,
        title="Work Surface Area",
        description="The usable work surface area that the product provides.",
        example="1 m2",
    )
    functional_storage_volume_m3: VolumeStr | None = pyd.Field(
        default=None,
        title="Functional Storage Volume",
        description="",
        example="1 m3",
    )
    functional_seating_capacity: pyd.NonNegativeInt | None = pyd.Field(
        default=None,
        title="Functional Seating Capacity",
        description="Intended number of individuals the product seats. This value is used in calculating impact per functional unit.",
        example=1,
    )
    installation_waste_factor: RatioFloat | None = pyd.Field(
        default=None,
        title="Installation Waste Factor",
        description="Typical increase in impacts to account for installation waste.",
        example=0.01,
    )

    # Nested specs:
    Chairs: ChairsV1 | None = None
    Countertops: CountertopsV1 | None = None
    DemountablePartitions: DemountablePartitionsV1 | None = None
    OtherFurnishings: OtherFurnishingsV1 | None = None
    StorageFurniture: StorageFurnitureV1 | None = None
    Tables: TablesV1 | None = None
    WorkSurfaces: WorkSurfacesV1 | None = None
    Workspaces: WorkspacesV1 | None = None



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
from openepd.compat.pydantic import pyd
from openepd.model.specs.base import BaseOpenEpdHierarchicalSpec
from openepd.model.specs.enums import (
    FoamType,
    InsulatingMaterial,
    InsulationIntendedApplication,
    MembraneRoofingReinforcement,
    RoofCoverBoardsFacing,
    RoofCoverBoardsMaterial,
)
from openepd.model.validation.numbers import RatioFloat
from openepd.model.validation.quantity import LengthMmStr, PressureMPaStr


class BituminousRoofingV1(BaseOpenEpdHierarchicalSpec):
    """
    Bituminous roofing.

    Premanufactured membrane roofing sheets consisting of asphalt, reinforcing layers, and in some cases a surfacing.
    """

    _EXT_VERSION = "1.0"


class SinglePlyEPDMV1(BaseOpenEpdHierarchicalSpec):
    """Ethylene propylene diene monomer (EPDM) rubber membrane."""

    _EXT_VERSION = "1.0"


class SinglePlyKEEV1(BaseOpenEpdHierarchicalSpec):
    """
    Ketone Ethylene Ester (KEE) roof membranes.

    Consist of PVC resin and KEE plasticizer, with KEE exceeding 50% of the polymer content by weight.
    """

    _EXT_VERSION = "1.0"


class SinglePlyOtherV1(BaseOpenEpdHierarchicalSpec):
    """Single ply other performance specification."""

    _EXT_VERSION = "1.0"


class SinglePlyPolyurethaneV1(BaseOpenEpdHierarchicalSpec):
    """Polyurethane liquid for flat roof waterproofing."""

    _EXT_VERSION = "1.0"


class SinglePlyPVCV1(BaseOpenEpdHierarchicalSpec):
    """Polyvinyl chloride (PVC) thermoplastic membrane."""

    _EXT_VERSION = "1.0"


class SinglePlyTPOV1(BaseOpenEpdHierarchicalSpec):
    """Thermoplastic Polyolefin (TPO) membrane."""

    _EXT_VERSION = "1.0"


class BlanketInsulationV1(BaseOpenEpdHierarchicalSpec):
    """Non-rigid insulation batts, blankets, and rolls."""

    _EXT_VERSION = "1.0"


class BlownInsulationV1(BaseOpenEpdHierarchicalSpec):
    """Loose-fill insulation for blow-in or closed cavity applications."""

    _EXT_VERSION = "1.0"


class BoardInsulationV1(BaseOpenEpdHierarchicalSpec):
    """Rigid insulation products including rigid foams, wood fiberboard insulation, and rigid mineral wool boards."""

    _EXT_VERSION = "1.0"

    # Own fields:
    compressive_strength: PressureMPaStr | None = pyd.Field(default=None, description="", example="1 MPa")


class FoamedInPlaceV1(BaseOpenEpdHierarchicalSpec):
    """Open and closed cell spray foam insulation."""

    _EXT_VERSION = "1.0"

    # Own fields:
    foam_type: FoamType | None = pyd.Field(default=None, description="", example="Open-Cell")


class SprayedInsulationV1(BaseOpenEpdHierarchicalSpec):
    """
    Spray-on insulation, such as spray-on cellulose.

    Foaming sprays are categorized separately under foamed-in-place.
    """

    _EXT_VERSION = "1.0"


class AirBarriersV1(BaseOpenEpdHierarchicalSpec):
    """Air Infiltration Barrier."""

    _EXT_VERSION = "1.0"


class MembraneRoofingV1(BaseOpenEpdHierarchicalSpec):
    """
    Membrane roofing.

    Built-up bituminous, modified bituminous, elastomeric, thermoplastic, fluid-applied, and hot-applied rubberized
    asphalt membrane roofing.
    """

    _EXT_VERSION = "1.0"

    # Own fields:
    thickness: LengthMmStr | None = pyd.Field(default=None, description="", example="10 mm")
    sri: float | None = pyd.Field(default=None, description="", example=2.3)
    total_recycled_content: RatioFloat | None = pyd.Field(default=None, description="", example=0.5, ge=0, le=1)
    post_consumer_recycled_content: RatioFloat | None = pyd.Field(default=None, description="", example=0.5, ge=0, le=1)
    reinforcement: MembraneRoofingReinforcement | None = pyd.Field(default=None, description="", example="Polyester")
    felt_backing: bool | None = pyd.Field(default=None, description="", example=True)
    nsf347: bool | None = pyd.Field(default=None, description="", example=True)
    vantage_vinyl: bool | None = pyd.Field(default=None, description="", example=True)

    # Nested specs:
    BituminousRoofing: BituminousRoofingV1 | None = None
    SinglePlyEPDM: SinglePlyEPDMV1 | None = None
    SinglePlyKEE: SinglePlyKEEV1 | None = None
    SinglePlyOther: SinglePlyOtherV1 | None = None
    SinglePlyPolyurethane: SinglePlyPolyurethaneV1 | None = None
    SinglePlyPVC: SinglePlyPVCV1 | None = None
    SinglePlyTPO: SinglePlyTPOV1 | None = None


class InsulationV1(BaseOpenEpdHierarchicalSpec):
    """Thermal insulation materials for use in construction."""

    _EXT_VERSION = "1.0"

    # Own fields:
    r_value: float | None = pyd.Field(default=None, description="", example=2.3)
    material: InsulatingMaterial | None = pyd.Field(default=None, description="", example="Mineral Wool")
    intended_application: list[InsulationIntendedApplication] | None = pyd.Field(
        default=None, description="", example=["Wall & General"]
    )
    thickness_per_declared_unit: LengthMmStr | None = pyd.Field(default=None, description="", example="10 mm")

    # Nested specs:
    BlanketInsulation: BlanketInsulationV1 | None = None
    BlownInsulation: BlownInsulationV1 | None = None
    BoardInsulation: BoardInsulationV1 | None = None
    FoamedInPlace: FoamedInPlaceV1 | None = None
    SprayedInsulation: SprayedInsulationV1 | None = None


class DampproofingAndWaterproofingV1(BaseOpenEpdHierarchicalSpec):
    """
    Dampproofing and waterproofing.

    Dampproofing, and built-up bituminous, sheet, fluid-applied, cementitious, reactive, and bentonite waterproofing.
    """

    _EXT_VERSION = "1.0"


class FlashingAndSheetMetalV1(BaseOpenEpdHierarchicalSpec):
    """Exposed sheet metal items, typically for drainage."""

    _EXT_VERSION = "1.0"


class JointProtectionV1(BaseOpenEpdHierarchicalSpec):
    """Preformed joint seals and sealants, expansion control joint cover assemblies."""

    _EXT_VERSION = "1.0"


class RoofCoverBoardsV1(BaseOpenEpdHierarchicalSpec):
    """
    Boards installed between the insulation and membrane layers on a roof system.

    It provides additional durability, fire protection, thermal, and vapor performance to a roof system, especially
    in low-slope, high foot traffic applications.
    """

    _EXT_VERSION = "1.0"

    # Own fields:
    material: RoofCoverBoardsMaterial | None = pyd.Field(default=None, description="", example="Gypsum Fiber")
    facing: list[RoofCoverBoardsFacing] | None = pyd.Field(default=None, description="", example=["Paper"])
    thickness: LengthMmStr | None = pyd.Field(default=None, description="", example="1 m")


class SteepSlopeRoofingV1(BaseOpenEpdHierarchicalSpec):
    """Roofing materials typically for slopes of 3:12 and greater."""

    _EXT_VERSION = "1.0"


class WeatherBarriersV1(BaseOpenEpdHierarchicalSpec):
    """Vapor retarders and sheet or membrane air barriers."""

    _EXT_VERSION = "1.0"


class ThermalMoistureProtectionV1(BaseOpenEpdHierarchicalSpec):
    """
    Thermal moisture protection.

    Broad category of materials whose function is to provide moisture and thermal protection between spaces (e.g.,
    between the exterior and interior of a building).
    """

    _EXT_VERSION = "1.0"

    # Nested specs:
    AirBarriers: AirBarriersV1 | None = None
    MembraneRoofing: MembraneRoofingV1 | None = None
    Insulation: InsulationV1 | None = None
    DampproofingAndWaterproofing: DampproofingAndWaterproofingV1 | None = None
    FlashingAndSheetMetal: FlashingAndSheetMetalV1 | None = None
    JointProtection: JointProtectionV1 | None = None
    RoofCoverBoards: RoofCoverBoardsV1 | None = None
    SteepSlopeRoofing: SteepSlopeRoofingV1 | None = None
    WeatherBarriers: WeatherBarriersV1 | None = None



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
from openepd.compat.pydantic import pyd
from openepd.model.specs.base import BaseOpenEpdHierarchicalSpec
from openepd.model.specs.enums import GypsumFacing, GypsumFireRating, GypsumThickness
from openepd.model.validation.quantity import LengthMmStr, RFactorStr, validate_quantity_unit_factory


class CementitiousSheathingBoardV1(BaseOpenEpdHierarchicalSpec):
    """
    Cementitious sheathing board.

    Cementitious non-gypsum board used for sheathing exteriors, shaft walls, and interior walls/ceilings requiring
    moisture resistance.
    """

    _EXT_VERSION = "1.0"

    # Own fields:
    cement_board_thickness: LengthMmStr | None = pyd.Field(default=None, description="", example="10 mm")

    _cement_board_thickness_is_quantity_validator = pyd.validator("cement_board_thickness", allow_reuse=True)(
        validate_quantity_unit_factory("m")
    )


class GypsumSheathingBoardV1(BaseOpenEpdHierarchicalSpec):
    """
    Cementitious sheathing board.

    Gypsum board used for sheathing exteriors, shaft walls, and interior walls/ceilings requiring moisture resistance.
    """

    _EXT_VERSION = "1.1"

    # Own fields:
    fire_rating: GypsumFireRating | None = pyd.Field(default=None, description="", example="-")
    thickness: GypsumThickness | None = pyd.Field(default=None, description="", example="9 mm")
    facing: GypsumFacing | None = pyd.Field(default=None, description="", example="Paper")

    r_factor: RFactorStr | None = pyd.Field(default=None, description="", example="1 RSI")

    flame_spread_astm_e84: int | None = pyd.Field(default=None, description="", example=3)
    smoke_production_astm_e84: int | None = pyd.Field(default=None, description="", example=3)
    surface_abrasion_d4977: int | None = pyd.Field(default=None, description="", example=3)
    indentation_d5420: int | None = pyd.Field(default=None, description="", example=3)
    soft_body_impact_e695: int | None = pyd.Field(default=None, description="", example=3)
    hard_body_impact_c1929: int | None = pyd.Field(default=None, description="", example=3)

    mold_resistant: bool | None = pyd.Field(default=None, description="", example=True)
    foil_backing: bool | None = pyd.Field(default=None, description="", example=True)
    moisture_resistant: bool | None = pyd.Field(default=None, description="", example=True)
    abuse_resistant: bool | None = pyd.Field(default=None, description="", example=True)

    _gypsum_thickness_is_quantity_validator = pyd.validator("thickness", allow_reuse=True)(
        validate_quantity_unit_factory("m")
    )
    _gypsum_r_factor_is_quantity_validator = pyd.validator("r_factor", allow_reuse=True)(
        validate_quantity_unit_factory("RSI")
    )


class SheathingV1(BaseOpenEpdHierarchicalSpec):
    """
    Sheathing.

    Boards or panels used in floor, wall and roof assemblies as a surface onto which other materials can be applied.
    """

    _EXT_VERSION = "1.0"

    # Nested specs:
    CementitiousSheathingBoard: CementitiousSheathingBoardV1 | None = None
    GypsumSheathingBoard: GypsumSheathingBoardV1 | None = None



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
from openepd.compat.pydantic import pyd
from openepd.model.base import BaseOpenEpdSchema
from openepd.model.common import Amount


class Category(BaseOpenEpdSchema):
    """DTO for Category model, recursive."""

    id: str = pyd.Field(description="Category short ID (readable unique string)")
    name: str = pyd.Field(description="Category display name (user-friendly)")
    short_name: str = pyd.Field(description="Category short user-friendly name")
    openepd_hierarchical_name: str = pyd.Field(
        "Special form of hierarchical category ID where the >> is hierarchy separator"
    )
    masterformat: str | None = pyd.Field(description="Default category code in Masterformat")
    description: str | None = pyd.Field(description="Category verbose description")
    declared_unit: Amount | None = pyd.Field(description="Declared unit of category, for example 1 kg")
    subcategories: list["Category"] = pyd.Field(
        description="List of subcategories. This makes categories tree-like structure"
    )



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
__all__ = (
    "IntumescentFireproofingRangeV1",
    "SprayFireproofingRangeV1",
    "AppliedFireproofingRangeV1",
    "FirestoppingRangeV1",
    "FireAndSmokeProtectionRangeV1",
)

# NB! This is a generated code. Do not edit it manually. Please see src/openepd/model/specs/README.md


from openepd.compat.pydantic import pyd
from openepd.model.specs.base import BaseOpenEpdHierarchicalSpec
from openepd.model.specs.enums import (
    IntumescentFireproofingMaterialType,
    SprayFireproofingDensity,
    SprayFireproofingMaterialType,
)
from openepd.model.validation.quantity import AmountRangeLengthMm


class IntumescentFireproofingRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Fireproofing material applied to structural materials, which swells as a result of heat exposure.

    As a result it increases in volume and decreasing in density.

    Range version.
    """

    _EXT_VERSION = "1.0"

    material_type: list[IntumescentFireproofingMaterialType] | None = pyd.Field(default=None, description="")


class SprayFireproofingRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Spray fireproofing.

    A passive fire protection system that reduces the rate of temperature increase in concrete or steel during a fire.

    Range version.
    """

    _EXT_VERSION = "1.0"

    material_type: list[SprayFireproofingMaterialType] | None = pyd.Field(default=None, description="")
    density: list[SprayFireproofingDensity] | None = pyd.Field(default=None, description="")


class AppliedFireproofingRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Fireproofing material applied to structural materials.

    Materials include: cement aggregate, cementitious, magnesium-oxychloride, intumescent, magnesium cement, mineral
    fiber, and mineral fiber fireproofing products.

    Range version.
    """

    _EXT_VERSION = "1.0"

    thickness: AmountRangeLengthMm | None = pyd.Field(default=None, description="")
    IntumescentFireproofing: IntumescentFireproofingRangeV1 | None = None
    SprayFireproofing: SprayFireproofingRangeV1 | None = None


class FirestoppingRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Seals and protects openings and joints in fire rate assemblies.

    Typically sealants, sprays, and caulks.

    Range version.
    """

    _EXT_VERSION = "1.0"


class FireAndSmokeProtectionRangeV1(BaseOpenEpdHierarchicalSpec):
    """
    Fire and smoke protection.

    General category of materials whose function is to provide protection of materials, spaces, and occupants from
    fire and smoke damage.

    Range version.
    """

    _EXT_VERSION = "1.0"

    AppliedFireproofing: AppliedFireproofingRangeV1 | None = None
    Firestopping: FirestoppingRangeV1 | None = None



#
#  Copyright 2025 by C Change Labs Inc. www.c-change-labs.com
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
from typing import Literal, TypeAlias, overload

from requests import Response

from openepd.api.base_sync_client import BaseApiMethodGroup
from openepd.api.common import StreamingListResponse, paging_meta_from_v1_api
from openepd.api.dto.common import BaseMeta, OpenEpdApiResponse
from openepd.api.dto.meta import PagingMetaMixin
from openepd.api.utils import encode_path_param
from openepd.model.industry_epd import IndustryEpd, IndustryEpdPreview, IndustryEpdRef


class IndustryEpdApi(BaseApiMethodGroup):
    """API methods for Industry EPD."""

    @overload
    def get_by_openxpd_uuid(self, uuid: str, with_response: Literal[True]) -> tuple[IndustryEpd, Response]: ...

    @overload
    def get_by_openxpd_uuid(self, uuid: str, with_response: Literal[False] = False) -> IndustryEpd: ...

    def get_by_openxpd_uuid(self, uuid: str, with_response: bool = False) -> IndustryEpd | tuple[IndustryEpd, Response]:
        """
        Get Industry EPD by OpenEPD UUID.

        :param uuid: OpenEPD UUID
        :param with_response: whether to return just object or with response
        :return: IEPD or IEPD with response depending on param with_response
        :raise ObjectNotFound: if Industry EPD is not found
        """
        response = self._client.do_request("get", f"/industry_epds/{uuid}")
        if with_response:
            return IndustryEpd.parse_obj(response.json()), response
        return IndustryEpd.parse_obj(response.json())

    @overload
    def create(self, iepd: IndustryEpd, with_response: Literal[True]) -> tuple[IndustryEpdRef, Response]: ...

    @overload
    def create(self, iepd: IndustryEpd, with_response: Literal[False] = False) -> IndustryEpdRef: ...

    def create(
        self, iepd: IndustryEpd, with_response: bool = False
    ) -> IndustryEpdRef | tuple[IndustryEpdRef, Response]:
        """
        Create an Industry EPD.

        :param iepd: Industry EPD
        :param with_response: return the response object together with the EPD
        :return: Industry EPD or Industry EPD with HTTP Response object depending on parameter
        """
        response = self._client.do_request(
            "post",
            "/industry_epds",
            json=iepd.to_serializable(exclude_unset=True, exclude_defaults=True, by_alias=True),
        )
        content = response.json()
        if with_response:
            return IndustryEpdRef.parse_obj(content), response
        return IndustryEpdRef.parse_obj(content)

    @overload
    def edit(self, iepd: IndustryEpd, with_response: Literal[True]) -> tuple[IndustryEpdRef, Response]: ...

    @overload
    def edit(self, iepd: IndustryEpd, with_response: Literal[False] = False) -> IndustryEpdRef: ...

    def edit(self, iepd: IndustryEpd, with_response: bool = False) -> IndustryEpdRef | tuple[IndustryEpdRef, Response]:
        """
        Edit an Industr EPD.

        :param iepd: IndustryEpd
        :param with_response: return the response object together with the IEPD
        :return: IEPD or IEPD with HTTP Response object depending on parameter
        """
        iepd_id = iepd.id
        if not iepd_id:
            raise ValueError("The ID must be set to edit a IndustryEpd.")

        response = self._client.do_request(
            "put",
            f"/industry_epds/{encode_path_param(iepd_id)}",
            json=iepd.to_serializable(exclude_unset=True, exclude_defaults=True, by_alias=True),
        )
        content = response.json()
        if with_response:
            return IndustryEpdRef.parse_obj(content), response
        return IndustryEpdRef.parse_obj(content)

    @overload
    def list_raw(
        self, page_num: int, page_size: int, with_response: Literal[False] = False
    ) -> list[IndustryEpdPreview]: ...

    @overload
    def list_raw(
        self, page_num: int, page_size: int, with_response: Literal[True]
    ) -> tuple[list[IndustryEpdPreview], Response]: ...

    def list_raw(
        self, page_num: int = 1, page_size: int = 10, with_response: bool = False
    ) -> list[IndustryEpdPreview] | tuple[list[IndustryEpdPreview], Response]:
        """
        List industry epds.

        :param page_num: page number
        :param page_size: page size
        :param with_response: whether to return just object or with response
        :return: list of IEPDs or list of IEPDs with response depending on param with_response
        """
        response = self._client.do_request(
            "get",
            "/industry_epds",
            params=dict(
                page_number=page_num,
                page_size=page_size,
            ),
        )
        data = [IndustryEpdPreview.parse_obj(o) for o in response.json()]
        if with_response:
            return data, response
        return data

    def list(self, page_size: int | None = None) -> StreamingListResponse[IndustryEpdPreview]:
        """
        List IndustryEpds.

        :param page_size: page size, None for default
        :return: streaming list of IEPDs
        """

        def _get_page(p_num: int, p_size: int) -> IndustryEpdListResponse:
            data_list, response = self.list_raw(page_num=p_num, page_size=p_size, with_response=True)
            return IndustryEpdListResponse(
                payload=data_list, meta=IndustryEpdSearchMeta(paging=paging_meta_from_v1_api(response))
            )

        return StreamingListResponse[IndustryEpdPreview](_get_page, page_size=page_size)


class IndustryEpdSearchMeta(PagingMetaMixin, BaseMeta):
    """Metadata for EPD Search endpoint."""

    pass


IndustryEpdListResponse: TypeAlias = OpenEpdApiResponse[list[IndustryEpdPreview], IndustryEpdSearchMeta]
